package com.eleadmin.chat.controller;

import com.eleadmin.chat.domain.entity.ChatEntity;
import com.eleadmin.chat.domain.from.ChatAddForm;
import com.eleadmin.chat.domain.from.ChatQueryForm;
import com.eleadmin.chat.domain.vo.ChatVO;
import com.eleadmin.chat.service.ChatService;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.core.web.PageResult;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 聊天记录表 Controller
 *
 * @Author 王益亭
 * @Date 2025-03-03 10:05:18
 * @Copyright gxrd
 */
@RestController
@RequestMapping("/api/chat")
public class ChatController extends BaseController {

    @Resource
    private ChatService chatService;

    /**
     * 分页查询聊天记录
     *
     * @param queryForm 查询表单
     * @return 分页结果
     */
    @GetMapping("/queryPage")
    @PreAuthorize("hasAuthority('chat:query')")
    public ApiResult<PageResult<ChatVO>> queryPage(ChatQueryForm queryForm) {
        queryForm.setFromUserId(getLoginUserId());
        queryForm.setToUserId(getLoginUserId());
        return success(chatService.queryPage(queryForm));
    }

    /**
     * 添加聊天记录
     *
     * @param body 聊天记录实体
     * @return 操作结果
     */
    @PostMapping("/add")
    @PreAuthorize("hasAuthority('chat:add')")
    public ApiResult<String> add(@RequestBody Map<String, List<ChatAddForm>> body) {
        chatService.saveChat(body.get("list"));
        return ApiResult.ok();
    }

    /**
     * 提交到下一步
     *
     * @param body 聊天记录实体
     * @return 操作结果
     */
    @PostMapping("/submit/{id}")
    @PreAuthorize("hasAuthority('chat:add')")
    public ApiResult<String> submit(@PathVariable Integer id) {
        chatService.submit(id);
        return ApiResult.ok();
    }

    /**
     * 删除聊天记录及其关联数据
     *
     * @param id 聊天记录ID
     * @return 操作结果
     */
    @DeleteMapping("/{id}")
    @PreAuthorize("hasAuthority('chat:delete')")
    public ApiResult<String> delete(@PathVariable Integer id) {
        // 调用新增的删除方法
        chatService.deleteChatAndRelatedData(id);
        return ApiResult.ok();
    }
}