package com.eleadmin.chat.controller;

import com.eleadmin.chat.domain.entity.ReplyEntity;
import com.eleadmin.chat.domain.from.ReplyQueryForm;
import com.eleadmin.chat.domain.vo.ReplyVO;
import com.eleadmin.chat.service.ReplyService;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.core.web.PageResult;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 回复消息表 Controller
 *
 * @Author 王益亭
 * @Date 2025-03-03 10:05:18
 * @Copyright gxrd
 */
@RestController
@RequestMapping("/api/reply")
public class ReplyController extends BaseController {

    @Resource
    private ReplyService replyService;

    /**
     * 分页查询回复消息
     *
     * @param queryForm 查询表单
     * @return 分页结果
     */
    @GetMapping("/queryPage")
    //@PreAuthorize("hasAuthority('reply:query')")
    public ApiResult<List<ReplyVO>> queryPage(ReplyQueryForm queryForm) {
        return success(replyService.queryPage(queryForm));
    }

    /**
     * 根据ID查询回复消息
     *
     * @param id 回复消息ID
     * @return 回复消息详情
     */
    @GetMapping("/{id}")
    //@PreAuthorize("hasAuthority('reply:query')")
    public ApiResult<ReplyEntity> getById(@PathVariable Integer id) {
        return success(replyService.getById(id));
    }

    /**
     * 添加回复消息
     *
     * @param replyEntity 回复消息实体
     * @return 操作结果
     */
    @PostMapping("/add")
    //@PreAuthorize("hasAuthority('reply:add')")
    public ApiResult<String> add(@RequestBody ReplyEntity replyEntity) {
        replyService.save(replyEntity);
        return ApiResult.ok();
    }

    /**
     * 更新回复消息
     *
     * @param replyEntity 回复消息实体
     * @return 操作结果
     */
    @PostMapping("/update")
    //@PreAuthorize("hasAuthority('reply:update')")
    public ApiResult<String> update(@RequestBody ReplyEntity replyEntity) {
        replyService.updateById(replyEntity);
        return ApiResult.ok();
    }

    /**
     * 删除回复消息
     *
     * @param id 回复消息ID
     * @return 操作结果
     */
    @DeleteMapping("/{id}")
    //@PreAuthorize("hasAuthority('reply:delete')")
    public ApiResult<String> delete(@PathVariable Integer id) {
        replyService.removeById(id);
        return ApiResult.ok();
    }
}