package com.eleadmin.chat.service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eleadmin.chat.dao.ChatDao;
import com.eleadmin.chat.dao.ReplyDao;
import com.eleadmin.chat.domain.entity.ChatEntity;
import com.eleadmin.chat.domain.entity.ChatToUserEntity;
import com.eleadmin.chat.domain.entity.ReplyEntity;
import com.eleadmin.chat.domain.from.ChatAddForm;
import com.eleadmin.chat.domain.from.ChatQueryForm;
import com.eleadmin.chat.domain.vo.ChatVO;
import com.eleadmin.common.core.utils.LoginUtil;
import com.eleadmin.common.core.web.PageParam;
import com.eleadmin.common.core.web.PageResult;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class ChatService extends ServiceImpl<ChatDao, ChatEntity> {

    private final ReplyService replyService;

    public PageResult<ChatVO> queryPage(ChatQueryForm queryForm) {
        PageParam<ChatEntity, ChatQueryForm> pageParam = new PageParam<>(queryForm);
        List<ChatVO> list = baseMapper.queryPage(pageParam, queryForm);
        return new PageResult<>(list, pageParam.getTotal());
    }

    @Transactional
    public void saveChat(List<ChatAddForm> list) {
        ChatEntity chat = new ChatEntity();
        chat.setIndex(1);
        chat.setFromUserId(LoginUtil.getLoginUser().getUserId());
        save(chat);

        List<ReplyEntity> entities = list.stream().map(it -> {
            ReplyEntity entity = new ReplyEntity();
            entity.setChatId(chat.getId());
            entity.setName(it.getName());
            entity.setContent(it.getContent());
            entity.setIndex(it.getIndex());
            entity.setToUserId(it.getUser());
            return entity;
        }).toList();

        replyService.saveBatch(entities);
    }

    // 新增：删除聊天记录及其关联数据
    @Transactional
    public void deleteChatAndRelatedData(Integer chatId) {
        // 删除关联的 to_user 数据
        // 删除关联的 reply 数据
        replyService.deleteByChatId(chatId);
        // 删除聊天记录本身
        removeById(chatId);
    }

    @Transactional
    public void submit(Integer id) {
        ReplyEntity byId = replyService.getById(id);
        ReplyEntity entity = new ReplyEntity();
        entity.setId(id);
        entity.setCompleteTime(new Date());
        replyService.updateById(entity);

        ChatEntity chat = getById(byId.getChatId());

        ChatEntity chatEntity = new ChatEntity();
        chatEntity.setId(chat.getId());
        chatEntity.setIndex(chat.getIndex() + 1);
        updateById(chatEntity);
    }
}