package com.eleadmin.common.core.utils;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.exception.ExcelAnalysisException;
import com.alibaba.excel.metadata.data.ReadCellData;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.alibaba.excel.support.ExcelTypeEnum;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.channels.ClosedChannelException;
import java.util.*;

public class ExcelMapReader {

    /**
     * 读取所有Sheet数据，返回结构：Map<Sheet名称, List<行数据Map>> 
     */
    public Map<String, List<Map<String, Object>>> readAllSheets(File file) throws IOException {
        // 存储最终结果：Key=Sheet名称，Value=该Sheet的数据列表
        Map<String, List<Map<String, Object>>> result = new LinkedHashMap<>();


        try{
            FileInputStream fileInputStream = new FileInputStream(file);
            // 1. 获取所有Sheet信息
            List<ReadSheet> sheets = EasyExcel.read(fileInputStream).excelType(ExcelTypeEnum.XLS)
                    .build()
                    .excelExecutor()
                    .sheetList();
            fileInputStream.close();

            // 2. 遍历每个Sheet读取数据
            for (ReadSheet sheet : sheets) {
                // 每个Sheet的数据监听器
                SheetDataListener listener = new SheetDataListener();

                FileInputStream fileInputStream1 = new FileInputStream(file);
                // 读取当前Sheet（指定sheetNo和标题行）
                EasyExcel.read(fileInputStream1).excelType(ExcelTypeEnum.XLS)
                        .headRowNumber(1) // 假设第一行为标题行
                        .sheet(sheet.getSheetNo())
                        .registerReadListener(listener)
                        .doRead();
                fileInputStream1.close();
                // 将当前Sheet数据存入结果
                result.put(sheet.getSheetName(), listener.getDataList());
            }
        }catch (ExcelAnalysisException e) {
            Throwable cause = e.getCause();
            if (cause instanceof ClosedChannelException) {
                // 处理流关闭问题
                System.err.println("流已关闭：" + cause.getMessage());
            }
        }


        return result;
    }

    /**
     * 自定义Sheet数据监听器
     */
    private static class SheetDataListener implements ReadListener<Map<Integer, String>> {
        private List<Map<String, Object>> dataList = new ArrayList<>();
        private Map<Integer,  ReadCellData<?>> headerMap; // 存储列索引与标题的映射

        @Override
        public void invokeHead(Map<Integer, ReadCellData<?>> headMap, AnalysisContext context) {
            // 读取标题行（第一行）
            this.headerMap = headMap;
        }

        @Override
        public void invoke(Map<Integer, String> rowData, AnalysisContext context) {
            // 将索引映射的RowData转换为标题键的Map
            Map<String, Object> rowMap = new HashMap<>();
            for (Map.Entry<Integer, String> entry : rowData.entrySet()) {
                String header = headerMap.get(entry.getKey()).getStringValue();
                rowMap.put(header, entry.getValue());
            }
            dataList.add(rowMap);
        }

        @Override
        public void doAfterAllAnalysed(AnalysisContext context) {
            // 可在此处添加数据校验或后处理
        }

        public List<Map<String, Object>> getDataList() {
            return dataList;
        }
    }
}
