package com.eleadmin.common.core.utils;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;

public class LocalCache {
    private static final int MAX_CACHE_SIZE = 1000; // 最大缓存大小
    private static final TimedCache<String, Object> CACHE_MAP = CacheUtil.newTimedCache(MAX_CACHE_SIZE);

    /**
     * 存放一个缓存对象，默认保存时间半小时
     * @param cacheName
     * @param obj
     */
    public static void put(String cacheName, Object obj) {
        put(cacheName, obj, 60 * 60 / 2L);
    }

    /**
     * 存放一个缓存对象，保存时间为holdTime
     * @param cacheName
     * @param obj
     * @param holdTime
     */
    public static void put(String cacheName, Object obj, long holdTime) {
        CACHE_MAP.put(cacheName, obj, holdTime * 1000L);
    }

    public static void put(String cacheName, Object obj, int holdTime) {
        put(cacheName, obj, (long) holdTime);
    }

    /**
     * 取出一个缓存对象
     * @param cacheName
     * @return
     */
    public static Object get(String cacheName) {
        return CACHE_MAP.get(cacheName, false);
    }

    public static boolean containsKey(String cacheName) {
        return CACHE_MAP.containsKey(cacheName);
    }

    public static boolean isEmpty(String cacheName) {
        return !CACHE_MAP.containsKey(cacheName);
    }

    /**
     * 删除所有缓存
     */
    public static void removeAll() {
        CACHE_MAP.clear();
    }

    /**
     * 删除某个缓存
     * @param cacheName
     */
    public static void remove(String cacheName) {
        CACHE_MAP.remove(cacheName);
    }

    /**
     * 检查缓存对象是否存在，
     * 若不存在，则返回false
     * 若存在，检查其是否已过有效期，如果已经过了则删除该缓存并返回false
     * @param cacheName
     * @return
     */
    public static boolean checkCacheName(String cacheName) {
        return CACHE_MAP.containsKey(cacheName);
    }
}