package com.eleadmin.common.core.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class PhoneNumberUtil {
    /**
     * 提取字符串中符合中国电话号码规范的号码
     *
     * @param input 输入文本
     * @return 匹配到的电话号码列表
     */
    public static List<String> extractPhoneNumbers(String input) {
        List<String> results = new ArrayList<>();

        // 支持的电话号码正则表达式
        String phoneRegex = "1[3-9]\\d{9}"; // 手机号
        String fixedLineRegex = "0[1-9]\\d{1,2}-[1-9]\\d{5,7}"; // 固话（含区号）
        String internationalMobileRegex = "\\+86\\s?1[3-9]\\d{9}"; // 手机（国际格式）
        String internationalFixedLineRegex = "\\+86\\s?0[1-9]\\d{1,2}\\s?[1-9]\\d{5,7}"; // 固话（国际格式）
        String formattedMobileRegex = "1[3-9]\\d{3}[-\\s]?\\d{4}[-\\s]?\\d{4}"; // 带分隔符的手机号

        // 合并所有规则
        String fullRegex = String.join("|",
                phoneRegex,
                fixedLineRegex,
                internationalMobileRegex,
                internationalFixedLineRegex,
                formattedMobileRegex
        );

        Pattern pattern = Pattern.compile(fullRegex);
        Matcher matcher = pattern.matcher(input);

        while (matcher.find()) {
            results.add(matcher.group());
        }

        return results;
    }
}
