package com.eleadmin.common.flowStep.controller;

import com.eleadmin.common.core.annotation.Operation;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.flowStep.entity.Field;
import com.eleadmin.common.flowStep.entity.param.FieldParam;
import com.eleadmin.common.flowStep.service.IFieldService;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * 流程字段 前端控制器
 *
 * @author zg
 * @since 2025-03-24
 */
@RestController
@RequestMapping("/api/field")
public class FieldController extends BaseController {

    @Resource
    private IFieldService service;


    @Operation(summary = "分页查询")
    @GetMapping("/queryPage")
    @PreAuthorize("hasAuthority('field:query')")
    public ApiResult queryPage(FieldParam fieldParam) {
        return success(service.queryPage(fieldParam));
    }

    @Operation(summary = "详情")
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('field:query')")
    public ApiResult getById(@PathVariable Integer id) {
        return success(service.getById(id));
    }

    @Operation(summary = "新增")
    @PostMapping("")
    @PreAuthorize("hasAuthority('field:add')")
    public ApiResult save(@RequestBody Field entity) {
        entity.setCreateTime(LocalDateTime.now());
        entity.setCreateUser(getLoginUserId());
        return service.save(entity) ? success() : fail();
    }

    @Operation(summary = "修改")
    @PutMapping("")
    @PreAuthorize("hasAuthority('field:update')")
    public ApiResult update(@RequestBody Field entity) {
        entity.setCreateTime(null);
        entity.setCreateUser(null);
        return service.updateById(entity) ? success() : fail();
    }

    @Operation(summary = "删除")
    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('field:remove')")
    public ApiResult remove(@PathVariable Integer id) {
        return service.removeById(id) ? success() : fail();
    }
}
