package com.eleadmin.common.flowStep.controller;

import com.eleadmin.common.core.annotation.Operation;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.flowStep.entity.Flow;
import com.eleadmin.common.flowStep.entity.param.FlowParam;
import com.eleadmin.common.flowStep.service.IFlowService;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * 流程主表 前端控制器
 *
 * @author zg
 * @since 2025-03-24
 */
@RestController
@RequestMapping("/api/flow")
public class FlowController extends BaseController {

    @Resource
    private IFlowService service;



    @Operation(summary = "分页查询")
    @GetMapping("/queryPage")
    @PreAuthorize("hasAuthority('flow:query')")
    public ApiResult queryPage(FlowParam flowParam) {
        return success(service.queryPage(flowParam));
    }

    @Operation(summary = "详情")
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('flow:query')")
    public ApiResult getById(@PathVariable Integer id) {
        return success(service.getById(id));
    }

    @Operation(summary = "新增")
    @PostMapping("")
    @PreAuthorize("hasAuthority('flow:add')")
    public ApiResult save(@RequestBody Flow flow) {
        flow.setCreateTime(LocalDateTime.now());
        flow.setCreateUser(getLoginUserId());
        return service.save(flow) ? success() : fail();
    }

    @Operation(summary = "修改")
    @PutMapping("")
    @PreAuthorize("hasAuthority('flow:update')")
    public ApiResult update(@RequestBody Flow flow) {
        flow.setCreateTime(null);
        flow.setCreateUser(null);
        return service.updateById(flow) ? success() : fail();
    }

    @Operation(summary = "删除")
    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('flow:remove')")
    public ApiResult remove(@PathVariable Integer id) {
        return service.removeById(id) ? success() : fail();
    }

}
