package com.eleadmin.common.flowStep.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eleadmin.common.core.annotation.Operation;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.flowStep.entity.Step;
import com.eleadmin.common.flowStep.entity.param.FieldParam;
import com.eleadmin.common.flowStep.entity.param.StepParam;
import com.eleadmin.common.flowStep.service.IFieldService;
import com.eleadmin.common.flowStep.service.IStepService;
import com.eleadmin.common.system.service.RoleService;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * 步骤表 前端控制器
 *
 * @author zg
 * @since 2025-03-24
 */
@RestController
@RequestMapping("/api/step")
public class StepController extends BaseController {

    @Resource
    private IStepService service;
    @Resource
    private RoleService roleService;
    @Resource
    private IFieldService fieldService;

    @Operation(summary = "分页查询")
    @GetMapping("/queryPage")
    @PreAuthorize("hasAuthority('field:query')")
    public ApiResult queryPage(StepParam param) {
        return success(service.queryPage(param));
    }

    @Operation(summary = "详情")
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('field:query')")
    public ApiResult getById(@PathVariable Integer id) {
        return success(service.getById(id));
    }

    @Operation(summary = "角色下拉")
    @GetMapping("getRoles")
    @PreAuthorize("hasAuthority('field:add')")
    public ApiResult getRole() {
       return success( roleService.list());
    }

    @Operation(summary = "字段下拉")
    @GetMapping("getFields")
    @PreAuthorize("hasAuthority('field:add')")
    public ApiResult getFields(FieldParam fieldParam) {
        return success( fieldService.getFields(fieldParam));
    }

    @Operation(summary = "新增")
    @PostMapping("")
    @PreAuthorize("hasAuthority('field:add')")
    public ApiResult save(@RequestBody Step entity) {
        if (entity.getStepNumber() == null){
            Long count = service.count(Wrappers.lambdaQuery(Step.class).eq(Step::getFlowId, entity.getFlowId()));
            entity.setStepNumber(Integer.parseInt(count.toString()) + 1);
        }
        entity.setCreateTime(LocalDateTime.now());
        entity.setCreateUser(getLoginUserId());
        return service.save(entity) ? success() : fail();
    }

    @Operation(summary = "修改")
    @PutMapping("")
    @PreAuthorize("hasAuthority('field:update')")
    public ApiResult update(@RequestBody Step entity) {
        entity.setCreateTime(null);
        entity.setCreateUser(null);
        return service.updateById(entity) ? success() : fail();
    }

    @Operation(summary = "删除")
    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('field:remove')")
    public ApiResult remove(@PathVariable Integer id) {
        return service.removeById(id) ? success() : fail();
    }
}
