package com.eleadmin.crawler;

import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.XmlUtil;
import cn.hutool.crypto.Mode;
import cn.hutool.crypto.Padding;
import cn.hutool.crypto.digest.MD5;
import cn.hutool.crypto.symmetric.AES;
import com.eleadmin.flowable.service.FlowableService;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.jsoup.Connection;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
@Service
@RequiredArgsConstructor
public class CrawlerService {

    Pattern userTokenPattern = Pattern.compile("\\$userToken = \"[^\"]+\"");
    Pattern clsIDPattern = Pattern.compile("bdGrid\\.clsID = \"[^\"]+\"");
    Pattern appIDPattern = Pattern.compile("bdGrid\\.appID = \"[^\"]+\"");
    Pattern userIdPattern = Pattern.compile("id {3}= \"[^\"]+\"");

    private Map<String, String> cookie = null;

    private final String username = "houw5243";
    private final String password = "龘aa3a1fbc369a33f800edc67b437dbf4f";
    private final String agent = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 10.0; WOW64; Trident/7.0; .NET4.0C; .NET4.0E;  QIHU 360ENT)";

    private String userId = null;
    private String userToken = null;
    private String appId = null;
    private String clsId = null;

    private final HttpClient client = HttpClient.newHttpClient();
    private final HttpRequest loginUri = HttpRequest.newBuilder().header("User-Agent", agent).uri(URI.create("http://10.55.6.80/MWWebSite/PROJECT-HOME/NARI-OMS/login/login.jsp")).build();

    private final AES aes = new AES(Mode.CBC, Padding.PKCS5Padding, "0102030405060708".getBytes(StandardCharsets.UTF_8), "0102030405060708".getBytes(StandardCharsets.UTF_8));


    private final TaskService taskService;
    private final FlowableService flowableService;


    @PostConstruct
    @Scheduled(cron = "0 0 9,17 * * ?")
    public void loginUser() {
        try {
            CompletableFuture.runAsync(() -> {
                login();
                getLoginUser();
            });
        } catch (Exception e) {
            log.warn("登录失败");
        }
    }

    @Scheduled(cron = "0 0/30 * * * ?")
    public void execute() {
        CompletableFuture.runAsync(() -> {
            List<Task> tasks = taskService.createTaskQuery().taskName("图模归档").active().list();
            for (Task task : tasks) {
                Map<String, Object> variables = task.getTaskLocalVariables();
                @SuppressWarnings("unchecked")
                List<String> omsNumber = (List<String>) flowableService.getVariable(task.getProcessInstanceId(), "omsNumber");
                Map<String, String> omsStatus;
                if (variables.containsKey("omsStatus")) {
                    omsStatus = (Map<String, String>) variables.get("omsStatus");
                } else {
                    omsStatus = new HashMap<>();
                }

                for (String number : omsNumber) {
                    String status = omsStatus.get(number);
                    if (StrUtil.isBlank(status)) {
                        omsStatus.put(number, list(number) ? "已执行" : "");
                    }
                }
                taskService.setVariableLocal(task.getId(), "omsStatus", omsStatus);
            }
        });
    }

    private void loginDev() {
        HashMap<String, String> t = new HashMap<>();
        t.put("redirectHome", "false");
        t.put("JSESSIONID", "Ut78JA6U8MacI3ZOGsOQ1cl657PTAKY7wmLSmt-VDVHNdqimHxTA!-1473293525!1747986943636");
        t.put("loginName", "houw5243");
        this.cookie = t;
    }

    @SneakyThrows
    private void login() {
        HttpResponse<String> response = client.send(loginUri, HttpResponse.BodyHandlers.ofString());
        if (response.statusCode() != 200 || response.body() == null) {
            return;
        }
        Document parse = Jsoup.parse(response.body());
        String curToken = parse.body().getElementById("curToken").val();
        System.out.println(curToken);

        Map<String, String> map = new HashMap<>();
        map.put("userName", username);
        map.put("password", password);
        map.put("curToken", "龘" + MD5.create().digestHex(curToken));

        Connection.Response execute = Jsoup.connect("http://10.55.6.80/MWWebSite/handlers/securitymodel/login?redirectUrl=%2FMWWebSite%2FPROJECT-HOME%2FNARI-OMS%2Fconsole%2FDefault.jsp").data(map).method(Connection.Method.POST).execute();
        this.cookie = execute.cookies();
    }

    @SneakyThrows
    private void getLoginUser() {
        Connection.Response response = Jsoup.connect("http://10.55.6.80/MWWebSite/PROJECT-HOME/NARI-OMS/business/GGYY/TYCX/TYCX.jsp?clsID=0E5F82FE-4F7B-4D12-A76B-17FA01535E3A").method(Connection.Method.POST).cookies(this.cookie).execute();
        if (response.statusCode() != 200) {
            return;
        }
        Elements scripts = response.parse().select("script");
        for (Element script : scripts) {
            String html = script.html();
            String userId = getUserId(html);
            if (userId != null) {
                this.userId = userId;
            }
            String userToken = getUserToken(html);
            if (userToken != null) {
                this.userToken = userToken;
            }
            String appId = getAppId(html);
            if (appId != null) {
                this.appId = appId;
            }
            String clsId = getClsID(html);
            if (clsId != null) {
                this.clsId = clsId;
            }
        }

//        System.out.println("userId: " + userId);
//        System.out.println("userToken: " + userToken);
//        System.out.println("appId: " + appId);
//        System.out.println("clsId: " + this.clsId);
    }

    private String getUserToken(String html) {
        return getString(userTokenPattern, html);
    }

    private String getUserId(String html) {
        return getString(userIdPattern, html);
    }

    private String getAppId(String html) {
        return getString(appIDPattern, html);
    }

    private String getClsID(String html) {
        return getString(clsIDPattern, html);
    }

    private String getString(Pattern userIdPattern, String html) {
        Matcher matcher = userIdPattern.matcher(html);
        if (matcher.find()) {
            return matcher.group(0).split("=")[1].replaceAll("\"", "").trim();
        }
        return null;
    }

    private boolean list(String omsNumber) {
        try {
            long l = System.currentTimeMillis();
            String format = initXmlParam(omsNumber);

            String code = aes.encryptBase64(format);

            Connection.Response execute = Jsoup.connect("http://10.55.6.48:7001/MWBusinessModel/xmlrpc?rnd=" + l)
                    .header("Accept-Encoding", "gzip deflate")
                    .header("Accept", "*/*")
                    .header("Content-Type", "application/x-www-form-urlencoded")
                    .header("Accept-Language", "zh-cn")
                    .header("Pragma", "no-cache")
                    .requestBody(code)
                    .postDataCharset("UTF-8")
                    .method(Connection.Method.POST)
                    .referrer("http://10.55.6.80/MWWebSite/PROJECT-HOME/NARI-OMS/business/GGYY/TYCX/TYCX.jsp?clsID=0E5F82FE-4F7B-4D12-A76B-17FA01535E3A#")
                    .userAgent("Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 10.0; WOW64; Trident/7.0; .NET4.0C; .NET4.0E;  QIHU 360ENT)")
                    .execute();

            String body = execute.body();
            org.w3c.dom.Document document = XmlUtil.parseXml(body);
            org.w3c.dom.Element element = XmlUtil.getRootElement(document);
            org.w3c.dom.Element params = XmlUtil.getElement(element, "params");
            org.w3c.dom.Element param = XmlUtil.getElement(params, "param");
            org.w3c.dom.Element value = XmlUtil.getElement(param, "value");
            org.w3c.dom.Element struct = XmlUtil.getElement(value, "struct");
            List<org.w3c.dom.Element> member = XmlUtil.getElements(struct, "member");
            org.w3c.dom.Element resultValue = member.get(2);
            org.w3c.dom.Element value1 = XmlUtil.getElement(resultValue, "value");
            org.w3c.dom.Element array = XmlUtil.getElement(value1, "array");
            org.w3c.dom.Element businessDataCollection = XmlUtil.getElement(array, "businessDataCollection");
            List<org.w3c.dom.Element> businessDataList = XmlUtil.getElements(businessDataCollection, "businessData");

            Optional<org.w3c.dom.Element> first = businessDataList.stream().findFirst();

            if (first.isEmpty()) {
                log.warn("{}未查询到数据", omsNumber);
                return false;
            }

            String id = first.get().getAttribute("id");

            Document parse = Jsoup.connect("http://10.55.6.80/MWWebSite/PROJECT-HOME/NARI-OMS/business/DD/DDJHZY/PDSBXTBGTYLC/PDSBXTBGTYLC.jsp?clsID=" + clsId + "&appID=" + appId + "&prcdID=054F21D6-EF72-4882-9F41-33E9E88D4263&objID=" + id).cookies(cookie).get();
            return getResult(parse.html());
        } catch (Exception e) {
            log.error("{}查询异常", omsNumber, e);
            return false;
        }
    }

    private String initXmlParam(String omsNumber) {
        String xmlTemplate = """
                <methodCall>
                    <methodName>businessmodel.BusinessRunTimeService.batchPagingLoadBusinessData</methodName>
                    <context userID="${userId}" userName="${userName}" token="${userToken}" ip="10.186.42.172" agent="W"></context>
                    <params>
                        <param><value><string><![CDATA[${clsId}]]></string></value></param>
                        <param><value><string><![CDATA[${userId}]]></string></value></param>
                        <param><value><string><![CDATA[${appId}]]></string></value></param>
                        <param><value><null/></value></param>
                        <param><value><string><![CDATA[<CLSFilters><CLSFilter clsID="${clsId}" filterString="1=1 AND SQDBH Like '${sqdbh}' AND SBDWID='${sbdwid}'" userControl="F" enabled="T"/></CLSFilters>]]></string></value></param>
                        <param><value><null/></value></param>
                        <param><value><null/></value></param>
                        <param><value><i4>100</i4></value></param>
                        <param><value><i4>0</i4></value></param>
                    </params>
                </methodCall>
                """;

        Map<String, Object> params = new HashMap<>();
        params.put("userId", this.userId);
        params.put("userName", this.username);
        params.put("userToken", this.userToken);
        params.put("clsId", this.clsId);
        params.put("appId", this.appId);
        params.put("sbdwid", "633F65E0-164A-4A8C-BD4E-D08F2BB585B5-00115");
        params.put("sqdbh", "%" + omsNumber + "%");

        return replacePlaceholders(xmlTemplate, params);
    }

    private String replacePlaceholders(String template, Map<String, Object> params) {
        for (Map.Entry<String, Object> entry : params.entrySet()) {
            template = template.replaceAll("\\$\\{" + entry.getKey() + "}", entry.getValue().toString());
        }
        return template;
    }


    @SneakyThrows
    private boolean getResult(String html) {
        String selector = "#PDZX_table_cnt .tabItem > tbody > tr > td:last-child";
        Element result = Jsoup.parse(html).selectFirst(selector);
        if (result != null) {
            log.warn("=============================================");
            log.warn(result.html());
            return result.html().contains("已执行");
        } else {
            log.warn("Element not found with selector: {}", selector);
        }

        return false;
    }

}
