package com.eleadmin.distributiondeviceinfo.controller;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.resource.ResourceUtil;
import com.eleadmin.common.core.annotation.Operation;
import com.eleadmin.common.core.annotation.Tag;
import com.eleadmin.common.core.utils.SmartResponseUtil;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.distributiondeviceinfo.domain.entity.DistributionDeviceInfoEntity;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoLinkAddForm;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoLinkQueryForm;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoLinkUpdateForm;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoQueryForm;
import com.eleadmin.distributiondeviceinfo.service.DistributionDeviceInfoService;
import com.eleadmin.powerdebug.domain.entity.PowerDebugEntity;
import com.eleadmin.common.system.entity.Role;
import com.eleadmin.common.system.param.IdParam;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * 配电设备新投异动图模维护信息表 Controller
 *
 * @Author 王益亭
 * @Date 2025-02-28 14:39:52
 * @Copyright gxrd
 */

@RestController
@Tag(name = "配电设备新投异动图模维护信息表")
public class DistributionDeviceInfoController extends BaseController {

    @Resource
    private DistributionDeviceInfoService distributionDeviceInfoService;

    @Operation(summary = "分页查询 @author 王益亭")
    @PostMapping("/api/distributionDeviceInfo/queryPage")
    @PreAuthorize("hasAnyAuthority('outageWork:query','common:flow')")
    public ApiResult queryPage(@RequestBody @Valid DistributionDeviceInfoQueryForm queryForm) {
//        List<Integer> collect = getLoginUser().getRoles().stream().map(Role::getRoleId).toList();
//        if (!collect.contains(1)) {
//            queryForm.setCreateUser(getLoginUser().getUserId());
//        }
        return success(distributionDeviceInfoService.querySimplePage(queryForm));
    }

    @Operation(summary = "查询关联信息 @author 王益亭")
    @PostMapping("/api/distributionDeviceInfo/link")
    @PreAuthorize("hasAnyAuthority('outageWork:query','common:flow')")
    public ApiResult<PowerDebugEntity> queryLink(@RequestBody @Valid DistributionDeviceInfoLinkQueryForm queryForm) {
        return success(distributionDeviceInfoService.queryLink(queryForm));
    }

    @Operation(summary = "根据ID查询 @author 王益亭")
    @PostMapping("/api/distributionDeviceInfo/{id}")
    @PreAuthorize("hasAnyAuthority('outageWork:query','common:flow')")
    public ApiResult getById(@PathVariable Integer id) {
        return success(distributionDeviceInfoService.getById(id));
    }

    @Operation(summary = "导入Excel @author 王益亭")
    @PostMapping("/api/distributionDeviceInfo/importExcel/{outageId}")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:import','common:flow')")
    public ApiResult<?> importExcel(@RequestParam("file") MultipartFile file, @PathVariable Integer outageId) {
        List<Integer> collect = getLoginUser().getRoles().stream().map(Role::getRoleId).toList();
        Integer commitStatus = null;
        if (collect.contains(1)) {
            commitStatus = 1;
        }
        distributionDeviceInfoService.importFromExcel(file, getLoginUserId(), commitStatus, outageId);
        return success();
    }
    @Operation(summary = "导入Excel查看")
    @PostMapping("/api/distributionDeviceInfo/importExcelCheck")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:import','common:flow')")
    public ApiResult<?> importExcelCheck(@RequestParam("file") MultipartFile file) {
        return success(distributionDeviceInfoService.importFromExcelCheck(file));
    }

    @Operation(summary = "查询excel数据                 ")
    @GetMapping("/api/distributionDeviceInfo/getExcelInfo")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:import','common:flow')")
    public ApiResult getExcel(Integer id) throws IOException {
        return success(distributionDeviceInfoService.getExcel(id));
    }

    @Operation(summary = "模板 @author 王益亭")
    @GetMapping("/api/distributionDeviceInfo/model")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:import','common:flow')")
    public void model(HttpServletResponse response) throws IOException {
        // 设置下载消息头
        SmartResponseUtil.setDownloadFileHeader(response, "新投异动模板", null);
        IoUtil.copy(ResourceUtil.getStream("classpath:/model/model.xlsx"), response.getOutputStream());
    }

    @Operation(summary = "添加 @author 王益亭")
    @PostMapping("/api/distributionDeviceInfo/add")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:add','common:flow')")
    public ApiResult<String> add(@RequestBody DistributionDeviceInfoLinkAddForm addForm) {
        return distributionDeviceInfoService.add(addForm);
    }

    @Operation(summary = "提交")
    @PostMapping("/api/distributionDeviceInfo/commit")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:add','common:flow')")
    public ApiResult commit(@RequestBody @Valid IdParam idParam) {
        distributionDeviceInfoService.commit(idParam);
        return success();
    }


    /*@Operation(summary = "提交任务")
    @PostMapping("/api/distributionDeviceInfo/submit/{id}")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:add")
    public ApiResult<?> submit(@PathVariable Integer id) {
        distributionDeviceInfoService.submit(id);
        return success();
    }*/

    @Operation(summary = "撤销提交")
    @PostMapping("/api/distributionDeviceInfo/repealCommit")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:add','common:flow')")
    public ApiResult repealCommit(@RequestBody @Valid IdParam idParam) {
//        distributionDeviceInfoService.repealCommit(idParam);
        return success();
    }

    @Operation(summary = "更新 @author 王益亭")
    @PostMapping("/api/distributionDeviceInfo/update")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:update','common:flow')")
    public ApiResult<String> update(@RequestBody DistributionDeviceInfoLinkUpdateForm updateForm) {
        List<Integer> collect = getLoginUser().getRoles().stream().map(Role::getRoleId).toList();
        Integer createUser = null;
        if (!collect.contains(1)) {
            createUser = getLoginUserId();
        }
        return distributionDeviceInfoService.update(updateForm, createUser);
    }

    @Operation(summary = "批量删除 @author 王益亭")
    @PostMapping("/api/distributionDeviceInfo/batchDelete")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:delete','common:flow')")
    public ApiResult<?> batchDelete(@RequestBody List<Integer> idList) {
        return distributionDeviceInfoService.batchDelete(idList);
    }

    @Operation(summary = "单个删除 @author 王益亭")
    @GetMapping("/api/distributionDeviceInfo/delete/{id}")
    @PreAuthorize("hasAnyAuthority('distributionDeviceInfo:delete','common:flow')")
    public ApiResult<?> batchDelete(@PathVariable Integer id) {
        return distributionDeviceInfoService.delete(id);
    }
}
