package com.eleadmin.distributiondeviceinfo.domain.form;

import com.eleadmin.common.core.annotation.Schema;
import com.eleadmin.distributiondeviceinfo.domain.entity.NewDevice;
import com.eleadmin.distributiondeviceinfo.domain.vo.AttachmentVO;
import com.fasterxml.jackson.annotation.JsonFormat;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.Data;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 配电设备新投异动图模维护信息表 新建表单
 *
 * @Author 王益亭
 * @Date 2025-02-28 14:39:52
 * @Copyright gxrd
 */

@Data
public class DistributionDeviceInfoAddForm {

    @Schema(description = "计划工作（送电）时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private LocalDateTime scheduledWorkTime;

    @Schema(description = "计划工作大馈线")
    private String planWorkFeeder;

    @Schema(description = "计划工作时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    @NotNull(message = "计划工作时间不能为空")
    private LocalDate planWorkTime;

    private Integer outageId;

    @Schema(description = "图模涉及大馈线名称")
    @NotBlank(message = "图模涉及大馈线名称不能为空")
    private String lineName;

    @Schema(description = "OMS新投异动编号")
    private List<String> omsNumber;

    @Schema(description = "主业工程名称")
    private String mainProjectName;

    @Schema(description = "主业工程编号")
    private String mainProjectCode;

    @Schema(description = "是否已推工程")
    private Integer isMainProject;

    @Schema(description = "运行单位推工程人")
    private String operatingUnitContact;

    @Schema(description = "客户工程名称")
    private String customerProjectName;

    @Schema(description = "客户工程编号")
    private String customerProjectCode;

    @Schema(description = "是否涉及主业实物ID扫码")
    private Integer involvesPhysicalId;

    @Schema(description = "运行单位扫码人")
    private String operatingUnitScanner;

    @Schema(description = "新增设备运行维护班组")
    private List<NewDevice> newEquipmentMaintenance;

    @Schema(description = "联系人及电话")
    private String contactPerson;

    @Schema(description = "专用设备进线电缆资产属性")
    private String specialEquipmentCableAsset;

    @Schema(description = "专用设备T接点开关、刀闸资产属性")
    private String specialEquipmentTapSwitchAsset;

    @Schema(description = "专用设备用户编号")
    private String specialEquipmentUserCode;

    @Schema(description = "大客户经理及联系方式")
    private String majorCustomerContact;

    @Schema(description = "附件")
    private List<AttachmentVO> attachment;
    @Schema(description = "提交状态")
    private Integer commitStatus;
    @Schema(description = "提交时间")
    private LocalDateTime commitTime;
    @Schema(description = "创建用户")
    private Integer createUser;

}