package com.eleadmin.distributiondeviceinfo.service;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.ClassUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.converters.AutoConverter;
import com.alibaba.excel.converters.Converter;
import com.alibaba.excel.converters.ConverterKeyBuild;
import com.alibaba.excel.converters.ReadConverterContext;
import com.alibaba.excel.enums.CellDataTypeEnum;
import com.alibaba.excel.metadata.Cell;
import com.alibaba.excel.metadata.data.ReadCellData;
import com.alibaba.excel.read.listener.ReadListener;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eleadmin.common.core.Constants;
import com.eleadmin.common.core.exception.BusinessException;
import com.eleadmin.common.core.utils.ExcelMapReader;
import com.eleadmin.common.core.utils.LoginUtil;
import com.eleadmin.common.core.utils.SmartBeanUtil;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.PageParam;
import com.eleadmin.common.core.web.PageResult;
import com.eleadmin.commonflow.from.FlowableParam;
import com.eleadmin.distributiondeviceinfo.dao.DistributionDeviceInfoDao;
import com.eleadmin.distributiondeviceinfo.domain.entity.DistributionDeviceInfoEntity;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoLinkAddForm;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoLinkQueryForm;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoLinkUpdateForm;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoQueryForm;
import com.eleadmin.distributiondeviceinfo.domain.vo.*;
import com.eleadmin.flowable.dto.FlowAssignDTO;
import com.eleadmin.flowable.service.FlowableService;
import com.eleadmin.outagework.domain.entity.OutageWorkEntity;
import com.eleadmin.outagework.domain.vo.OutageWorkVO;
import com.eleadmin.outagework.service.OutageWorkService;
import com.eleadmin.powerdebug.domain.entity.PowerDebugEntity;
import com.eleadmin.powerdebug.manager.PowerDebugManager;
import com.eleadmin.common.system.entity.Role;
import com.eleadmin.common.system.entity.User;
import com.eleadmin.common.system.param.IdParam;
import com.eleadmin.common.system.service.FileRecordService;
import jakarta.annotation.Resource;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 配电设备新投异动图模维护信息表 Service
 *
 * @Author 王益亭
 * @Date 2025-02-28 14:39:52
 * @Copyright gxrd
 */

@Service
@Slf4j
public class DistributionDeviceInfoService extends ServiceImpl<DistributionDeviceInfoDao, DistributionDeviceInfoEntity> implements IService<DistributionDeviceInfoEntity> {

    @Resource
    private DistributionDeviceInfoDao distributionDeviceInfoDao;

    @Resource
    private PowerDebugManager powerDebugManager;
    @Resource
    private FileRecordService fileRecordService;
    @Resource
    private FlowableService flowableService;
    @Resource
    private OutageWorkService outageWorkService;

    /**
     * 分页查询
     *
     * @param queryForm
     * @return
     */
    public PageResult<DistributionDeviceInfoVO> queryPage(DistributionDeviceInfoQueryForm queryForm) {
        PageParam page = new PageParam(queryForm);
        List<DistributionDeviceInfoVO> list = distributionDeviceInfoDao.queryPage(page, queryForm);
        PageResult<DistributionDeviceInfoVO> pageResult = new PageResult<>(list, page.getTotal());
        return pageResult;
    }


    public PageResult<DistributionDeviceSimpleVO> querySimplePage(DistributionDeviceInfoQueryForm queryForm) {
        PageParam page = new PageParam(queryForm);
        List<DistributionDeviceSimpleVO> list = distributionDeviceInfoDao.querySimplePage(page, queryForm);

        if (!list.isEmpty()) {
            List<OutageWorkEntity> entities = outageWorkService.lambdaQuery().select(OutageWorkEntity::getInspectionContent, OutageWorkEntity::getId).in(OutageWorkEntity::getId, list.stream().map(DistributionDeviceSimpleVO::getOutageId).collect(Collectors.toList())).list();

            Map<Integer, String> collect = entities.stream().collect(Collectors.toMap(OutageWorkEntity::getId, OutageWorkEntity::getInspectionContent));
            list.forEach(entity -> entity.setInspectionContent(collect.get(entity.getOutageId())));
        }

        return new PageResult<>(list, page.getTotal());
    }

    /**
     * 添加
     */
    public ApiResult<String> add(DistributionDeviceInfoLinkAddForm addForm) {
        DistributionDeviceInfoEntity de = SmartBeanUtil.copy(addForm.getLeft(), DistributionDeviceInfoEntity.class);
        de.setCreateTime(LocalDateTime.now());
        User loginUser = LoginUtil.getLoginUser();
       /* if (loginUser.getRoles().stream().map(Role::getRoleId).anyMatch(i -> i == 1)) {
            de.setCommitStatus(1);
            de.setCommitTime(LocalDateTime.now());
        }*/
        de.setCreateUser(loginUser.getUserId());

        setOutageId(de);
        try {
            distributionDeviceInfoDao.insert(de);

            if (de.getId() != null && addForm.getRight() != null) {
                PowerDebugEntity right = addForm.getRight();
                powerDebugManager.saveOrUpdate(right);
            }
        } catch (Exception e) {
            return new ApiResult<>(Constants.RESULT_ERROR_CODE, e.getMessage());
        }
        return ApiResult.ok();
    }

    private void setOutageId(DistributionDeviceInfoEntity de) {
        if (de.getOutageId() == null) {
            throw new BusinessException("没有outageId");
        }
        Long count = distributionDeviceInfoDao.selectCount(Wrappers.lambdaQuery(DistributionDeviceInfoEntity.class).eq(DistributionDeviceInfoEntity::getOutageId, de.getOutageId()));
        if (count > 0) {
            throw new BusinessException("该停电工作已存在信息表，请勿重复上传");
        }
        /*LocalDate time = de.getPlanWorkTime();
        LocalDateTime startTime = time.atStartOfDay();
        LocalDateTime endTime = time.atTime(23, 59, 59);
        // 匹配停电开始时间
        LambdaQueryWrapper<OutageWorkEntity> wrappers = Wrappers.lambdaQuery(OutageWorkEntity.class).eq(OutageWorkEntity::getId, de.getOutageId()).like(OutageWorkEntity::getInspectionContent, de.getPlanWorkFeeder()).between(OutageWorkEntity::getStartDate, startTime, endTime);
        long count = outageWorkService.count(wrappers);
        if (count == 0) {
            throw new BusinessException("填写的停电计划涉及大馈线或停电计划开始时间与停电计划的检修内容和计划工作时间不匹配");
        }
//        opt.ifPresent(it -> de.setOutageId(it.getId()));*/
    }

    /**
     * 更新
     *
     * @param updateForm
     * @return
     */
    public ApiResult<String> update(DistributionDeviceInfoLinkUpdateForm updateForm, Integer createUser) {
        DistributionDeviceInfoEntity entity = distributionDeviceInfoDao.selectById(updateForm.getLeft().getId());
        if (createUser != null && entity.getCommitStatus() >= 1) {
            throw new BusinessException("已提交，无法修改");
        }
        DistributionDeviceInfoEntity distributionDeviceInfoEntity = SmartBeanUtil.copy(updateForm.getLeft(), DistributionDeviceInfoEntity.class);
        try {
            if (entity.getOutageId() == null) {
                setOutageId(distributionDeviceInfoEntity);
            }
            distributionDeviceInfoDao.updateById(distributionDeviceInfoEntity);
            if (distributionDeviceInfoEntity.getId() != null && updateForm.getRight() != null) {
                powerDebugManager.saveOrUpdate(updateForm.getRight());
            }
        } catch (Exception e) {
            throw new BusinessException(e.getMessage());
        }

        return ApiResult.ok();
    }

    /**
     * 批量删除
     *
     * @param idList
     * @return
     */
    public ApiResult<?> batchDelete(List<Integer> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return ApiResult.ok();
        }

        distributionDeviceInfoDao.deleteByIds(idList);
        return ApiResult.ok();
    }


    @Transactional
    public void importFromExcel(MultipartFile file, Integer userId, Integer commitStatus, Integer outageId) {
        VerticalTableReadListener<DistributionDeviceInfoExcelVO> readListener = new VerticalTableReadListener<>(DistributionDeviceInfoExcelVO.class);
        try {
            EasyExcel.read(file.getInputStream(), readListener).sheet().headRowNumber(1).doRead();
            DistributionDeviceInfoExcelVO data = readListener.getData();
            DistributionDeviceInfoEntity entity = SmartBeanUtil.copy(data, DistributionDeviceInfoEntity.class);

            if (commitStatus != null) {
                entity.setCommitStatus(commitStatus);
                entity.setCommitTime(LocalDateTime.now());
            }
            entity.setCreateUser(userId);
            entity.setCreateTime(LocalDateTime.now());
            entity.setOutageId(outageId);

            setOutageId(entity);

            distributionDeviceInfoDao.insert(entity);
        } catch (Exception e) {
            throw new BusinessException(e.getMessage());
        }

    }

    public DistributionDeviceInfoEntity importFromExcelCheck(MultipartFile file) {
        VerticalTableReadListener<DistributionDeviceInfoExcelVO> readListener = new VerticalTableReadListener<>(DistributionDeviceInfoExcelVO.class);
        try {
            EasyExcel.read(file.getInputStream(), readListener).sheet().headRowNumber(1).doRead();
            DistributionDeviceInfoExcelVO data = readListener.getData();
            DistributionDeviceInfoEntity entity = SmartBeanUtil.copy(data, DistributionDeviceInfoEntity.class);
            return entity;
        } catch (Exception e) {
            throw new BusinessException(e.getMessage());
        }
    }

    public PowerDebugEntity queryLink(DistributionDeviceInfoLinkQueryForm queryForm) {
        return powerDebugManager.getOne(
                Wrappers.lambdaUpdate(PowerDebugEntity.class)
                        .eq(PowerDebugEntity::getPlannedTestDate, queryForm.getPlanWorkTime())
                        .eq(PowerDebugEntity::getLineName, queryForm.getPlanWorkFeeder())
                        .last("limit 1")
        );
    }

    public DistributionDeviceInfoUpdateVO getById(Integer id) {
        DistributionDeviceInfoUpdateVO vo = new DistributionDeviceInfoUpdateVO();
        var form = new DistributionDeviceInfoQueryForm();
        form.setPage(1L);
        form.setLimit(1L);
        form.setId(id);
        PageParam<DistributionDeviceInfoEntity, DistributionDeviceInfoQueryForm> page = new PageParam<>(form);
        distributionDeviceInfoDao.queryPage(page, form).stream().findFirst().ifPresent(vo::setLeft);
//        powerDebugManager.getOneOpt(Wrappers.lambdaQuery(PowerDebugEntity.class).eq(PowerDebugEntity::getDdiId, id).last("limit 1")).ifPresent(vo::setRight);
        return vo;
    }

    public ApiResult<?> delete(Integer id) {
        return distributionDeviceInfoDao.deleteById(id) > 0 ? ApiResult.ok() : ApiResult.fail();
    }

    @Transactional
    public void commit(IdParam idParam) {
        DistributionDeviceInfoEntity entity = distributionDeviceInfoDao.selectById(idParam.getId());
        if (entity.getCommitStatus() == 1) {
            throw new BusinessException("请勿重复提交");
        }

        DistributionDeviceInfoEntity distributionDeviceInfoEntity = new DistributionDeviceInfoEntity();
        distributionDeviceInfoEntity.setCommitStatus(1);
        distributionDeviceInfoEntity.setCommitTime(LocalDateTime.now());
        distributionDeviceInfoEntity.setId(idParam.getId());
        distributionDeviceInfoDao.updateById(distributionDeviceInfoEntity);

        submit(idParam);
    }

    @Transactional
    public void completeTask(DistributionDeviceInfoEntity entity, IdParam param) {
        Integer outageId = entity.getOutageId();
        OutageWorkVO workVO = outageWorkService.getById(outageId);
        String userId = LoginUtil.getLoginUser().getUserId().toString();
        FlowableParam flowableParam = new FlowableParam();
        flowableParam.setProcessInstanceId(workVO.getProcessInstanceId());
        flowableParam.setUserId(userId);
        flowableParam.setPage(1L);
        flowableParam.setLimit(1L);
        List<Task> tasks = flowableService.getTodoTasks(flowableParam).t2;
        if (tasks.isEmpty()) {
            return;
        }
        String id = tasks.get(0).getId();

        Map<String, Object> var = new HashMap<>();
        var.put("availableItems", entity.getOmsNumber());
        var.put("isTest", workVO.getWhetherDebug() == 1);

        boolean setNext = StringUtils.hasText(param.getCandidateId()) && StringUtils.hasText(param.getCandidateName());

        if (setNext) {
            var.put("tmhzz", param.getCandidateName());
        }

        flowableService.completeTask(id, userId, "提交", var);


        if (setNext) {
            flowableParam.setUserId(null);
            tasks = flowableService.getTodoTasks(flowableParam).t2;
            if (tasks.isEmpty()) {
                throw new BusinessException("获取下一步任务失败");
            }
            id = tasks.get(0).getId();
            flowableService.setAssignee(id, param.getCandidateId());
        }

    }

    public void repealCommit(IdParam idParam) {
        DistributionDeviceInfoEntity entity = distributionDeviceInfoDao.selectById(idParam.getId());
        if (entity.getCommitStatus() == 2) {
            throw new BusinessException("任务已完成，无法撤销");
        }
        DistributionDeviceInfoEntity distributionDeviceInfoEntity = new DistributionDeviceInfoEntity();
        distributionDeviceInfoEntity.setCommitStatus(0);
        distributionDeviceInfoEntity.setId(idParam.getId());
        distributionDeviceInfoDao.updateById(distributionDeviceInfoEntity);
    }

    public List<Map<String, List<Map<String, Object>>>> getExcel(Integer id) throws IOException {
        DistributionDeviceInfoEntity distributionDeviceInfoEntity = distributionDeviceInfoDao.selectById(id);
        if (CollectionUtils.isEmpty(distributionDeviceInfoEntity.getAttachment())) {
            throw new BusinessException("没有附件");
        }
        List<Map<String, List<Map<String, Object>>>> list = new ArrayList<>();
        for (AttachmentVO attachment : distributionDeviceInfoEntity.getAttachment()) {
            File file = new File(fileRecordService.getUploadDir(), attachment.getPath());
            ExcelMapReader excelMapReader = new ExcelMapReader();
            list.add(excelMapReader.readAllSheets(file));
        }
        return list;
    }

    @Transactional
    public void submit(IdParam param) {
        DistributionDeviceInfoEntity entity = distributionDeviceInfoDao.selectById(param.getId());
        if (entity.getCommitStatus() == 2) {
            throw new BusinessException("任务已完成，无法提交");
        }
        if (entity.getOutageId() == null) {
            throw new BusinessException("请先关联停电任务");
        }

        distributionDeviceInfoDao.update(Wrappers.lambdaUpdate(DistributionDeviceInfoEntity.class).eq(DistributionDeviceInfoEntity::getId, param.getId()).set(DistributionDeviceInfoEntity::getCommitStatus, 2));
        completeTask(entity, param);
    }


    @Slf4j
    public static class VerticalTableReadListener<T> implements ReadListener<Object> {

        private final T excelVO;
        private final Map<CharSequence, Tree<String>> fields;
        private Map<ConverterKeyBuild.ConverterKey, Converter<?>> converterMap;

        private String[] childHeadName = null;
        private ReadCellData<?> head;

        @SneakyThrows
        public VerticalTableReadListener(Class<T> clazz) {
            excelVO = clazz.getConstructor().newInstance();
            fields = initializeFields(excelVO.getClass());
        }

        private void processChildNodes(Tree<String> tree, Field field, List<ReadCellData<?>> values, AnalysisContext analysisContext) {
            if (childHeadName == null) {
                childHeadName = new String[]{values.get(0).getStringValue(), values.get(1).getStringValue()};
                return;
            }

            List<Tree<String>> children = tree.getChildren();

            Map<CharSequence, Tree<String>> collect = children.stream().collect(Collectors.toMap(Tree::getName, Function.identity()));

            try {
                Object instance = getRealType(field).getConstructor().newInstance();
                if (values.get(0) != null && !values.get(0).getType().equals(CellDataTypeEnum.EMPTY)) {
                    Tree<String> tree1 = collect.get(childHeadName[0]);
                    extracted(analysisContext, instance, values.get(0), (Field) tree1.get("field"), (ExcelProperty) tree1.get("anno"));
                }

                if (values.get(1) != null && !values.get(1).getType().equals(CellDataTypeEnum.EMPTY)) {
                    Tree<String> tree2 = collect.get(childHeadName[1]);
                    extracted(analysisContext, instance, values.get(1), (Field) tree2.get("field"), (ExcelProperty) tree2.get("anno"));
                }
                Object object = field.get(excelVO);
                if (object == null) {
                    field.setAccessible(true);
                    object = new ArrayList<>();
                    field.set(excelVO, object);
                }
                ((List<Object>) object).add(instance);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                throw new RuntimeException(e);
            }
        }

        // jdk基本类型List
        private void processChildNodes2(Field field, ReadCellData<?> value) {
            try {
                Object object = field.get(excelVO);
                if (object == null) {
                    field.setAccessible(true);
                    object = new ArrayList<>();
                    field.set(excelVO, object);
                }
                ((List<Object>) object).add(value.getStringValue());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                throw new RuntimeException(e);
            }
        }

        private void extracted(AnalysisContext analysisContext, Object vo, ReadCellData<?> cellData, Field field, ExcelProperty property) {
            try {
                Class<? extends Converter<?>> converterClazz = property.converter();
                Converter<?> converter;
                if (converterClazz == AutoConverter.class) {
                    converter = converterMap.get(ConverterKeyBuild.buildKey(field.getType(), cellData.getType()));
                } else {
                    converter = converterClazz.getConstructor().newInstance();
                }
                field.setAccessible(true);
                field.set(vo, converter.convertToJavaData(new ReadConverterContext<>(cellData, null, analysisContext)));
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                throw new RuntimeException(e);
            }
        }

        private static Map<CharSequence, Tree<String>> initializeFields(Class<?> clazz) {
            return Arrays.stream(ClassUtil.getDeclaredFields(clazz))
                    .map(it -> {
                        ExcelProperty annotation = it.getAnnotation(ExcelProperty.class);
                        Tree<String> node = TreeUtil.createEmptyNode("");
                        node.setName(annotation.value()[0]);
                        node.put("field", it);
                        node.put("anno", annotation);
                        Class<?> realType = getRealType(it);
                        boolean isList = it.getGenericType().getTypeName().contains("java.util.List");
                        if (!ClassUtil.isJdkClass(realType) && ClassUtil.getPackage(realType).startsWith("com.eleadmin")) {
                            List<Tree<String>> list = Arrays.stream(ClassUtil.getDeclaredFields(realType))
                                    .map(item -> {
                                        ExcelProperty anno = item.getAnnotation(ExcelProperty.class);
                                        Tree<String> nodes = TreeUtil.createEmptyNode("");
                                        nodes.setName(anno.value()[0]);
                                        nodes.put("field", item);
                                        nodes.put("anno", anno);
                                        nodes.setParent(node);
                                        return nodes;
                                    }).toList();
                            node.setChildren(list);
                            node.put("isBaseList", "0");
                        } else if (ClassUtil.isJdkClass(realType) && isList) {
                            node.put("isBaseList", "1");
                        }
                        return node;
                    })
                    .collect(Collectors.toMap(Tree::getName, it -> it));
        }

        private static Class<?> getRealType(Field field) {
            Type genericType = field.getGenericType();
            Class<?> actualType;
            if (genericType instanceof ParameterizedType parameterizedType) {
                Type[] actualTypeArguments = parameterizedType.getActualTypeArguments();
                actualType = (Class<?>) actualTypeArguments[0];
            } else {
                actualType = field.getType();
            }
            return actualType;
        }

        @Override
        public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        }

        public T getData() {
            return excelVO;
        }

        @Override
        public void invoke(Object o, AnalysisContext analysisContext) {
            if (converterMap == null) {
                converterMap = analysisContext.currentReadHolder().converterMap();
            }
            Map<Integer, Cell> cellMap = analysisContext.readRowHolder().getCellMap();
            if (cellMap.size() < 3) {
                log.warn("Insufficient cells in the row, skipping processing.");
                return;
            }

            ReadCellData<?> key = (ReadCellData<?>) cellMap.get(0);
            List<ReadCellData<?>> values = Arrays.asList((ReadCellData<?>) cellMap.get(1), (ReadCellData<?>) cellMap.get(2));
            if (key.getStringValue() != null) {
                head = key;
            }
            if (head == null) {
                log.warn("Head cell is null, skipping processing.");
                return;
            }

            Tree<String> tree = fields.get(head.getStringValue());
            if (tree == null) {
                log.warn("Field not found for head value: {}", head.getStringValue());
                return;
            }

            Field field = (Field) tree.get("field");
            field.setAccessible(true);

            if ("1".equals(tree.get("isBaseList"))) {
                processChildNodes2(field, (ReadCellData<?>) cellMap.get(1));
            } else if (tree.hasChild()) {
                processChildNodes(tree, field, values, analysisContext);
            } else {
                if (values.get(0) == null || values.get(0).getType().equals(CellDataTypeEnum.EMPTY)) {
                    return;
                }
                extracted(analysisContext, excelVO, values.get(0), field, (ExcelProperty) tree.get("anno"));
            }
        }
    }
}
