package com.eleadmin.faLine.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eleadmin.common.core.annotation.Operation;
import com.eleadmin.common.core.utils.SmartExcelUtil;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.faLine.domain.dto.FaExcelDTO;
import com.eleadmin.faLine.domain.dto.FaLineParam;
import com.eleadmin.faLine.domain.entity.FaLine;
import com.eleadmin.faLine.service.IFaLineService;
import com.eleadmin.outagework.dao.OutageWorkDao;
import com.eleadmin.outagework.domain.entity.OutageWorkEntity;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.SneakyThrows;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.stream.Stream;

/**
 * fa线路 前端控制器
 *
 * @author zg
 * @since 2025-03-20
 */
@RestController
@RequestMapping("/api/faLine")
public class FaLineController extends BaseController {

    @Resource
    private IFaLineService service;
    @Resource
    private OutageWorkDao outageWorkDao;


    @Operation(summary = "分页查询")
    @GetMapping("/queryPage")
    @PreAuthorize("hasAuthority('faLine:query')")
    public ApiResult queryPage(FaLineParam faLineParam) {
        return success(service.queryPage(faLineParam));
    }

    @Operation(summary = "详情")
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('faLine:query')")
    public ApiResult getById(@PathVariable Integer id) {
        return success(service.getById(id));
    }

    @Operation(summary = "新增")
    @PostMapping("")
    @PreAuthorize("hasAuthority('faLine:add')")
    public ApiResult save(@RequestBody FaLine faLine) {
        setWhetherFaLine(faLine, 1);
        return service.save(faLine) ? success() : fail();
    }

    @Operation(summary = "修改")
    @PutMapping("")
    @PreAuthorize("hasAuthority('faLine:update')")
    public ApiResult update(@RequestBody FaLine faLine) {
        setWhetherFaLine(faLine, 1);
        return service.updateById(faLine) ? success() : fail();
    }

    @Operation(summary = "删除")
    @DeleteMapping("/{id}")
    @PreAuthorize("hasAuthority('faLine:remove')")
    public ApiResult remove(@PathVariable List<Integer> id) {
        List<FaLine> faLine = service.listByIds(id);
        for (FaLine line : faLine) {
            setWhetherFaLine(line, 0);
        }
        return service.removeByIds(id) ? success() : fail();
    }

    @SneakyThrows
    @Operation(summary = "导入")
    @GetMapping("/model")
    @PreAuthorize("hasAuthority('faLine:import')")
    public void model(HttpServletResponse response) {
        SmartExcelUtil.exportExcel(response, "FA线路模板", "sheet1", FaExcelDTO.class, Collections.emptyList());
    }

    @SneakyThrows
    @Operation(summary = "导入")
    @PostMapping("/import")
    @PreAuthorize("hasAuthority('faLine:import')")
    @Transactional
    public ApiResult<?> importData(@RequestParam MultipartFile file) {
        List<FaExcelDTO> list = SmartExcelUtil.readExcelWithHeaderValidation(file.getInputStream(), FaExcelDTO.class);
        if (list.isEmpty()) {
            return success();
        }
        List<FaLine> lines = new ArrayList<>(list.stream().flatMap(it -> {
            String[] split = it.getLine().split("[，、]");
            if (split.length == 0 || split[0].isEmpty()) {
                return Stream.empty(); // 返回空的 Stream 而不是 null
            }
            return Arrays.stream(split)
                    .filter(line -> !line.isEmpty()) // 过滤掉空字符串
                    .map(line -> {
                        FaLine faLine = new FaLine();
                        faLine.setLineName(line);
                        return faLine;
                    });
        }).toList());

        if (lines.isEmpty()) {
            return success();
        }

        //判断在数据库中是否存在
        List<FaLine> faLines = service.list(Wrappers.lambdaQuery(FaLine.class).in(FaLine::getLineName, lines.stream().map(FaLine::getLineName).toList()));
        lines.removeIf(it -> faLines.stream().anyMatch(faLine -> faLine.getLineName().equals(it.getLineName())));

        service.saveBatch(lines);

        for (FaLine line : lines) {
            setWhetherFaLine(line, 1);
        }

        return success();
    }

    private void setWhetherFaLine(FaLine line, int i) {
        outageWorkDao.update(Wrappers.lambdaUpdate(OutageWorkEntity.class).like(OutageWorkEntity::getInspectionContent, line.getLineName()).set(OutageWorkEntity::getWhetherFaline, i));
    }
}
