package com.eleadmin.flowable.listener;

import lombok.RequiredArgsConstructor;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.ExecutionListener;
import org.flowable.engine.delegate.JavaDelegate;
import org.flowable.task.api.Task;
import org.flowable.task.service.delegate.DelegateTask;
import org.flowable.task.service.delegate.TaskListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Component
@RequiredArgsConstructor
public class SubProcessCompleteListener implements JavaDelegate {

    private final RuntimeService runtimeService;
    private final TaskService taskService;

    @Override
    public void execute(DelegateExecution execution) {
        String mainProcessId = (String) execution.getVariable("mainProcessInstanceId");

        @SuppressWarnings("unchecked")
        Map<String,String[]> startedSubProcesses = (Map<String,String[]>) runtimeService.getVariable(mainProcessId, "startedSubProcesses");
        @SuppressWarnings("unchecked")
        List<String> completedSubProcesses = (List<String>) runtimeService.getVariable(mainProcessId, "completedSubProcesses");

        if (completedSubProcesses == null) {
            completedSubProcesses = new ArrayList<>();
        }

        completedSubProcesses.add(execution.getProcessInstanceId());

        runtimeService.setVariable(mainProcessId, "completedSubProcesses", completedSubProcesses);

        runtimeService.setVariable(mainProcessId, "isTest", false);

        List<String> finalCompletedSubProcesses = completedSubProcesses;
        startedSubProcesses.keySet().stream().filter(it -> !it.equals(execution.getProcessInstanceId()) && !finalCompletedSubProcesses.contains(it)).forEach(it -> runtimeService.setVariable(it, "isTest", false));

        if (completedSubProcesses.size() == startedSubProcesses.size()) {
            // 所有子流程已完成，继续执行主流程
            Task task = taskService.createTaskQuery().processInstanceId(mainProcessId).taskName("等待子流程完成").singleResult();
            if (task != null) {
                taskService.complete(task.getId());
            }
        }
    }
}



