package com.eleadmin.flowable.listener;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eleadmin.common.core.exception.BusinessException;
import com.eleadmin.common.core.utils.LoginUtil;
import com.eleadmin.common.system.entity.User;
import com.eleadmin.common.system.param.UserParam;
import com.eleadmin.common.system.service.UserService;
import com.eleadmin.crawler.CrawlerService;
import lombok.RequiredArgsConstructor;
import org.flowable.engine.impl.el.FixedValue;
import org.flowable.task.service.delegate.DelegateTask;
import org.flowable.task.service.delegate.TaskListener;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Stream;

@RequiredArgsConstructor
@Component
public class TaskCreateListener implements TaskListener {

    private FixedValue type;

    private FixedValue typeIdStr;

    private final UserService userService;
    private final CrawlerService crawlerService;

    @Override
    public void notify(DelegateTask delegateTask) {
        // 获取待办人（支持候选组/候选人）
        List<String> assignees = getTaskAssignees(delegateTask);

        if (assignees.isEmpty()) {
            throw new BusinessException("未获取到下一步骤的候选人");
        }
        if (assignees.size() == 1) {
            delegateTask.setAssignee(assignees.get(0));
        }else {
            delegateTask.addCandidateUsers(assignees);
        }

        if (delegateTask.getName().equals("图模归档")) {
            crawlerService.loginUser();
            crawlerService.execute();
        }
        
    }

    private List<String> getTaskAssignees(DelegateTask delegateTask) {
        //0-用户/ 1-部门/ 2-角色/ 3-由前端下拉菜单传递
        final String expressionText = typeIdStr.getExpressionText();
        return switch (Integer.parseInt(type.getExpressionText())) {
            case 0 -> Collections.singletonList(expressionText);
            case 1 -> userService.list(Wrappers.lambdaQuery(User.class).eq(User::getOrganizationId, LoginUtil.getLoginUser().getOrganizationId())).stream().map(t -> t.getUserId().toString()).toList();
            case 2 -> {
                //String variable = delegateTask.getVariable("mainOperatingDepartment", String.class).substring(0, 2);
                UserParam.UserParamBuilder builder = UserParam.builder().roleId(Integer.parseInt(expressionText));
                /*if (delegateTask.getName().equals("资料收集")) {
                    builder.organizationName(variable);
                }*/
                List<String> list = new java.util.ArrayList<>(userService.listRel(builder.build()).stream().map(it -> it.getUserId().toString()).toList());
                if (delegateTask.getName().equals("资料收集")) {
                    list.addAll(userService.listRel(UserParam.builder().roleId(1).build()).stream().map(it -> it.getUserId().toString()).toList());
                }
                yield list;
            }
            default -> Collections.emptyList();
        };
    }

}

