package com.eleadmin.national_holidays.controller;

import cn.hutool.core.date.DateUtil;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.core.web.PageParam;
import com.eleadmin.common.core.web.PageResult;
import com.eleadmin.national_holidays.domain.entity.NationalHoliday;
import com.eleadmin.national_holidays.domain.from.NationalHolidayFrom;
import com.eleadmin.national_holidays.service.NationalHolidayService;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * 国家节假日及调休安排控制器
 */
@RestController
@RequestMapping("/api/national-holidays")
@RequiredArgsConstructor
public class NationalHolidayController extends BaseController {

    private final NationalHolidayService nationalHolidayService;

    /**
     * 分页查询国家节假日信息
     */
    @GetMapping("/queryPage")
    public ApiResult<PageResult<NationalHoliday>> queryPage(NationalHolidayFrom nationalHolidayFrom) {
        PageParam<NationalHoliday, NationalHolidayFrom> param = new PageParam<>(nationalHolidayFrom);
        nationalHolidayService.page(param, param.getOrderWrapper());
        return success(new PageResult<>(param.getRecords(), param.getTotal()));
    }

    /**
     * 根据ID获取国家节假日详细信息
     */
    @GetMapping("/{id}")
    public ApiResult<NationalHoliday> get(@PathVariable Long id) {
        return success(nationalHolidayService.getById(id));
    }

    /**
     * 新增国家节假日信息
     */
    @PostMapping
    public ApiResult<Boolean> create(@RequestBody NationalHoliday nationalHoliday) {
        return success(nationalHolidayService.save(nationalHoliday));
    }

    /**
     * 更新国家节假日信息
     */
    @PutMapping("/{id}")
    public ApiResult<Boolean> update(@PathVariable Long id, @RequestBody NationalHoliday nationalHoliday) {
        nationalHoliday.setId(id);
        return success(nationalHolidayService.updateById(nationalHoliday));
    }

    /**
     * 删除国家节假日信息
     */
    @DeleteMapping("/{id}")
    public ApiResult<Boolean> delete(@PathVariable Long id) {
        return success(nationalHolidayService.removeById(id));
    }

    /**
     * 导入国家节假日信息
     */
    @PostMapping("/import")
    public ApiResult<?> importFromExcel(@RequestParam("file") MultipartFile file) {
        nationalHolidayService.importFromExcel(file);
        return success();
    }

    /**
     * 导入国家节假日信息
     */
    @GetMapping("/subWorkday")
    public ApiResult<?> subWorkday(@RequestParam String date, @RequestParam Integer workday) {
        return success(nationalHolidayService.subWorkday(DateUtil.parse(date), workday, false));
    }

}