package com.eleadmin.national_holidays.service;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eleadmin.common.core.utils.SmartExcelUtil;
import com.eleadmin.national_holidays.dao.NationalHolidayDao;
import com.eleadmin.national_holidays.domain.entity.NationalHoliday;
import com.eleadmin.national_holidays.domain.from.NationalHolidayExcelFrom;
import lombok.SneakyThrows;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collector;
import java.util.stream.Collectors;

/**
 * 国家节假日及调休安排服务实现类
 */
@Service
public class NationalHolidayService extends ServiceImpl<NationalHolidayDao, NationalHoliday> {


    @SneakyThrows
    @Transactional
    public void importFromExcel(MultipartFile file) {

        List<NationalHolidayExcelFrom> excelData = SmartExcelUtil.readExcelWithHeaderValidation(file.getInputStream(), NationalHolidayExcelFrom.class);

        List<NationalHoliday> nationalHolidayList = new ArrayList<>();

        for (NationalHolidayExcelFrom excelFrom : excelData) {

            DateUtil.range(excelFrom.getStartDate(), excelFrom.getEndDate(), DateField.DAY_OF_MONTH).forEachRemaining(date -> {
                NationalHoliday holiday = new NationalHoliday();
                holiday.setYear(DateUtil.year(date));
                holiday.setHolidayName(excelFrom.getHolidayName());
                holiday.setHolidayDate(date);
                holiday.setIsWorkday(0);

                nationalHolidayList.add(holiday);
            });
            if (excelFrom.getSubstituteDates() != null && !excelFrom.getSubstituteDates().isEmpty()) {
                String[] substituteDateStrs = excelFrom.getSubstituteDates().split(",");
                for (String substituteDateStr : substituteDateStrs) {
                    Date substituteDate = DateUtil.parseDate(substituteDateStr.trim());

                    NationalHoliday holiday = new NationalHoliday();
                    holiday.setYear(DateUtil.year(substituteDate));
                    holiday.setHolidayName(excelFrom.getHolidayName());
                    holiday.setHolidayDate(substituteDate);
                    holiday.setIsWorkday(1);

                    nationalHolidayList.add(holiday);
                }
            }
        }

        saveBatch(nationalHolidayList);
    }

    public DateTime subWorkday(Date date, int workday, boolean isAdd) {
        int year = DateUtil.year(date);

        Map<Date, NationalHoliday> holidayMap = lambdaQuery().eq(NationalHoliday::getYear, year).list().stream().collect(Collectors.toMap(NationalHoliday::getHolidayDate, it -> it));

        for (int i = 0; i < workday; i++) {
            date = DateUtil.offsetDay(date, isAdd ? 1 : -1);

            NationalHoliday it = holidayMap.get(date);

            if (it == null) {
                if (isWeekend(date)) {
                    i--;
                }
            } else {
                if (it.getIsWorkday() == 0) {
                    i--;
                }
            }

        }
        return DateUtil.date(date);
    }

    /**
     * 判断给定日期是否是周末（周六或周日）
     *
     * @param date 日期
     * @return 如果是周末返回 true，否则返回 false
     */
    public static boolean isWeekend(Date date) {
        int dayOfWeek = DateUtil.dayOfWeek(date);
        return dayOfWeek == 1 || dayOfWeek == 7;
    }

}