package com.eleadmin.outagework.controller;

import com.eleadmin.common.core.annotation.Operation;
import com.eleadmin.common.core.annotation.Tag;
import com.eleadmin.common.core.utils.SmartExcelUtil;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.core.web.PageResult;
import com.eleadmin.outagework.domain.form.OutageWorkAddForm;
import com.eleadmin.outagework.domain.form.OutageWorkQueryForm;
import com.eleadmin.outagework.domain.form.OutageWorkUpdateForm;
import com.eleadmin.outagework.domain.vo.OutageWorkCancelInfoVO;
import com.eleadmin.outagework.domain.vo.OutageWorkCanelVo;
import com.eleadmin.outagework.domain.vo.OutageWorkExcelVO;
import com.eleadmin.outagework.domain.vo.OutageWorkFlowVo;
import com.eleadmin.outagework.service.OutageWorkService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * 对外停电工作记录表 Controller
 *
 * @Author 王益亭
 * @Date 2025-02-28 14:15:38
 * @Copyright gxrd
 */

@RestController
@Tag(name = "对外停电工作记录表")
public class OutageWorkController extends BaseController {

    @Resource
    private OutageWorkService outageWorkService;

    @Operation(summary = "分页查询 @author 王益亭")
    @PostMapping("/api/outageWork/queryPage")
//    @SaCheckPermission("outageWork:query")
    @PreAuthorize("hasAuthority('outageWork:query')")
    public ApiResult<PageResult> queryPage(@RequestBody @Valid OutageWorkQueryForm queryForm) {
        return success(outageWorkService.querySimplePage(queryForm));
    }

    @Operation(summary = "根据ID查询 @author 王益亭")
    @PostMapping("/api/outageWork/{id}")
//    // @SaCheckPermission("distributionDeviceInfo:query")
    @PreAuthorize("hasAnyAuthority('outageWork:query','common:flow')")
    public ApiResult getById(@PathVariable Integer id) {
        return success(outageWorkService.getById(id));
    }

    @Operation(summary = "导入Excel @author 王益亭")
    @PostMapping("/api/outageWork/importExcel/{isTemp}")
//    @SaCheckPermission("outageWork:import")
    @PreAuthorize("hasAuthority('outageWork:import')")
    public ApiResult<String> importExcel(@RequestParam("file") MultipartFile file,@PathVariable Integer isTemp) {
        return outageWorkService.importFromExcel(file,isTemp);
    }

    @Operation(summary = "对外停电模板 @author 王益亭")
    @GetMapping("/api/outageWork/model")
//    @SaCheckPermission("outageWork:import")
    @PreAuthorize("hasAuthority('outageWork:import')")
    public void model(HttpServletResponse response) throws IOException {
        SmartExcelUtil.exportExcel(response, "对外停电工作记录表","sheet", OutageWorkExcelVO.class,List.of(new OutageWorkExcelVO()));
    }

    @Operation(summary = "添加 @author 王益亭")
    @PostMapping("/api/outageWork/add")
//    @SaCheckPermission("outageWork:add")
    @PreAuthorize("hasAuthority('outageWork:add')")
    public ApiResult<String> add(@RequestBody @Valid OutageWorkAddForm addForm) {
        return outageWorkService.add(addForm);
    }

    @Operation(summary = "更新 @author 王益亭")
    @PostMapping("/api/outageWork/update")
    @PreAuthorize("hasAuthority('outageWork:update')")
    public ApiResult<String> update(@RequestBody @Valid OutageWorkUpdateForm updateForm) {
        return outageWorkService.update(updateForm);
    }

    @Operation(summary = "批量删除 @author 王益亭")
    @PostMapping("/api/outageWork/batchDelete")
    @PreAuthorize("hasAuthority('outageWork:remove')")
    public ApiResult<String> batchDelete(@RequestBody List<Integer> idList) {
        return outageWorkService.batchDelete(idList);
    }

    @Operation(summary = "单个删除 @author 王益亭")
    @GetMapping("/api/outageWork/delete/{id}")
    @PreAuthorize("hasAuthority('outageWork:remove')")
    public ApiResult<String> batchDelete(@PathVariable Integer id) {
        return outageWorkService.delete(id);
    }

    @Operation(summary = "查询流程详情")
    @GetMapping("/api/outageWork/getFlowList")
    @PreAuthorize("hasAuthority('outageWork:flow')")
    public ApiResult getFlowList(Integer id) {
        return success(outageWorkService.getFlowList(id));
    }

    @Operation(summary = "查询当前流程字段")
    @GetMapping("/api/outageWork/getFieldList")
    @PreAuthorize("hasAuthority('outageWork:flow')")
    public ApiResult getFieldList(Integer id) {
        return success(outageWorkService.getFieldList(id));
    }

    @Operation(summary = "走流程")
    @PostMapping("/api/outageWork/underwayFlow")
    @PreAuthorize("hasAuthority('outageWork:flow')")
    public ApiResult underwayFlow(@RequestBody @Validated OutageWorkFlowVo vo) {
        outageWorkService.underwayFlow(vo,getLoginUser());
        return success();
    }

    @Operation(summary = "取消")
    @PostMapping("/api/outageWork/cancel")
    @PreAuthorize("hasAuthority('outageWork:flow')")
    public ApiResult cancel(@RequestBody @Validated OutageWorkCanelVo vo) {
        outageWorkService.cancel(vo);
        return success();
    }

    @Operation(summary = "取消查询")
    @PostMapping("/api/outageWork/cancelInfo")
    @PreAuthorize("hasAuthority('outageWork:flow')")
    public ApiResult cancelInfo(@RequestBody @Validated OutageWorkCancelInfoVO outageWorkCancelInfoVO) {
        outageWorkService.cancelInfo(outageWorkCancelInfoVO);
        return success();
    }


    @Operation(summary = "根据停电ID查询图模信息 @author 王益亭")
    @PostMapping("/api/outageWork/disttributionDevice/{id}")
    @PreAuthorize("hasAnyAuthority('outageWork:query','common:flow')")
    public ApiResult getDisttributionDevice(@PathVariable Integer id) {
        return success(outageWorkService.getDisttributionDevice(id));
    }

    /**
     * 查询超期工作
     *
     */
    @Operation(summary = "查询超期工作")
    @GetMapping("/api/outageWork/overdueWork")
    @PreAuthorize("hasAuthority('outageWork:query')")
    public ApiResult<Long> getOverdueWork() {
        return success(outageWorkService.getOverdueWork());
    }
}
