package com.eleadmin.outagework.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eleadmin.common.core.exception.BusinessException;
import com.eleadmin.common.core.utils.*;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.PageParam;
import com.eleadmin.common.core.web.PageResult;
import com.eleadmin.common.system.entity.Organization;
import com.eleadmin.common.system.entity.Role;
import com.eleadmin.common.system.param.UserParam;
import com.eleadmin.common.system.service.OrganizationService;
import com.eleadmin.common.system.service.UserService;
import com.eleadmin.commonflow.from.FlowableParam;
import com.eleadmin.distributiondeviceinfo.dao.DistributionDeviceInfoDao;
import com.eleadmin.distributiondeviceinfo.domain.entity.DistributionDeviceInfoEntity;
import com.eleadmin.distributiondeviceinfo.domain.form.DistributionDeviceInfoQueryForm;
import com.eleadmin.distributiondeviceinfo.domain.vo.DistributionDeviceInfoUpdateVO;
import com.eleadmin.faLine.domain.entity.FaLine;
import com.eleadmin.faLine.service.IFaLineService;
import com.eleadmin.common.flowStep.entity.*;
import com.eleadmin.common.flowStep.entity.vo.StepTableVo;
import com.eleadmin.common.flowStep.service.*;
import com.eleadmin.flowable.service.FlowableService;
import com.eleadmin.flowable.vo.TaskInfo;
import com.eleadmin.outagework.dao.OutageWorkDao;
import com.eleadmin.outagework.domain.entity.OutageWorkEntity;
import com.eleadmin.outagework.domain.form.OutageWorkAddForm;
import com.eleadmin.outagework.domain.form.OutageWorkQueryForm;
import com.eleadmin.outagework.domain.form.OutageWorkUpdateForm;
import com.eleadmin.outagework.domain.vo.*;
import com.eleadmin.powerdebug.domain.entity.PowerDebugEntity;
import com.eleadmin.powerdebug.manager.PowerDebugManager;
import com.eleadmin.common.system.entity.User;
import com.eleadmin.powerdebug.service.PowerDebugService;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.flowable.task.api.Task;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 对外停电工作记录表 Service
 *
 * @Author 王益亭
 * @Date 2025-02-28 14:15:38
 * @Copyright gxrd
 */

@Service
@Slf4j
@RequiredArgsConstructor
public class OutageWorkService extends ServiceImpl<OutageWorkDao, OutageWorkEntity> implements IService<OutageWorkEntity> {


    private final OutageWorkDao outageWorkDao;
    private final IFaLineService faLineService;
    private final IFlowService flowService;
    private final IStepService stepService;
    private final IFieldService fieldService;
    private final IStepTableService stepTableService;
    private final IStepTableFieldService stepTableFieldService;
    private final DistributionDeviceInfoDao distributionDeviceInfoDao;
    private final FlowableService flowableService;
    private final PowerDebugService powerDebugService;


    /**
     * 分页查询
     *
     * @param queryForm
     * @return
     */
    public PageResult<OutageWorkVO> queryPage(OutageWorkQueryForm queryForm) {
        PageParam<OutageWorkEntity, OutageWorkQueryForm> param = new PageParam<>(queryForm);
        List<OutageWorkVO> list = outageWorkDao.queryPage(param, queryForm);
        return new PageResult<>(list, param.getTotal());
    }


    public PageResult<OutageWorkSimpleVO> querySimplePage(OutageWorkQueryForm queryForm) {
        PageParam<OutageWorkEntity, OutageWorkQueryForm> param = new PageParam<>(queryForm);
        List<OutageWorkSimpleVO> list = outageWorkDao.querySimplePage(param, queryForm);
        Map<String, List<TaskInfo>> tasks = flowableService.getCurrentTasks(list.stream().map(OutageWorkSimpleVO::getProcessInstanceId).filter(Objects::nonNull).collect(Collectors.toSet()));

        list.forEach(v -> v.setCurrentTask(tasks.get(v.getProcessInstanceId())));
        return new PageResult<>(list, param.getTotal());
    }

    /**
     * 添加
     */
    @Transactional
    public ApiResult<String> add(OutageWorkAddForm addForm) {
        OutageWorkEntity outageWorkEntity = SmartBeanUtil.copy(addForm, OutageWorkEntity.class);
        getSubLineName(outageWorkEntity);
        setFlow(outageWorkEntity);
        // 插入流程
        outageWorkDao.insert(outageWorkEntity);

        startFlow(outageWorkEntity, LoginUtil.getLoginUser().getUserId().toString());

        return ApiResult.ok();
    }

    public void startFlow(OutageWorkEntity outageWorkEntity, String userId) {
        if (outageWorkEntity.getIsNewProject() != 1) {
            return;
        }
        Map<String, Object> available = Map.of(
                "startedSubProcesses", new HashMap<String, String[]>(),
                "completedSubProcesses", new ArrayList<>()
        );
        String processId = flowableService.startProcess("common-flow", outageWorkEntity.getId().toString(), available);
        outageWorkDao.updateById(OutageWorkEntity.builder().id(outageWorkEntity.getId()).processInstanceId(processId).build());

        FlowableParam flowableParam = new FlowableParam();
        flowableParam.setProcessInstanceId(processId);
        flowableParam.setPage(1L);
        flowableParam.setLimit(1L);
        List<Task> tasks = flowableService.getTodoTasks(flowableParam).t2;
        String taskId = tasks.get(0).getId();

        Map<String, Object> var = new HashMap<>();
        var.put("endDateBegin", outageWorkEntity.getStartDate().atZone(ZoneId.systemDefault()).toInstant().getEpochSecond());

        var.put("endDateEnd", outageWorkEntity.getEndDate().atZone(ZoneId.systemDefault()).toInstant().getEpochSecond());
        var.put("inspectionContent", outageWorkEntity.getInspectionContent());
        var.put("powerOutageScope", org.springframework.util.StringUtils.hasText(outageWorkEntity.getPowerOutageScope()) ? outageWorkEntity.getPowerOutageScope() : "");
        var.put("lineName", org.springframework.util.StringUtils.hasText(outageWorkEntity.getLineName()) ? outageWorkEntity.getLineName() : "");

        flowableService.completeTask(taskId, userId, "发起申请", var); //Map.of("mainOperatingDepartment", outageWorkEntity.getMainOperatingDepartment())
    }

    private void setFlow(OutageWorkEntity outageWorkEntity) {
        Flow flow = flowService.getOne(Wrappers.lambdaQuery(Flow.class).eq(Flow::getEnable, 1));
        if (flow == null) {
            throw new BusinessException("没有启用的流程");
        }
        outageWorkEntity.setStepNumber(1);
        outageWorkEntity.setFlowId(flow.getId());
    }

    private void getSubLineName(OutageWorkEntity outageWorkEntity) {
        if (StringUtils.isNotBlank(outageWorkEntity.getInspectionContent())) {
            FaLine limit = faLineService.query().apply("'" + outageWorkEntity.getInspectionContent() + "'" + " like concat('%',line_name,'%')").last("limit 1").one();
            if (limit != null) {
                outageWorkEntity.setWhetherFaline(1);
            }

            List<String> substrings = ReUtil.findAllGroup1("<(.*?)>", outageWorkEntity.getInspectionContent());

            outageWorkEntity.setLineName(String.join("、", substrings));
        }
    }

    /**
     * 更新
     *
     * @param updateForm
     * @return
     */
    @Transactional
    public ApiResult<String> update(OutageWorkUpdateForm updateForm) {
        OutageWorkEntity outageWorkEntity = SmartBeanUtil.copy(updateForm, OutageWorkEntity.class);
        getSubLineName(outageWorkEntity);

//        List<Map<String, Object>> debugEntities = powerDebugService.listByContentAndTime(Collections.singletonList(outageWorkEntity));


        outageWorkDao.updateById(outageWorkEntity);

        OutageWorkEntity entity = outageWorkDao.selectOne(Wrappers.lambdaQuery(OutageWorkEntity.class).eq(OutageWorkEntity::getId, outageWorkEntity.getId()).select(OutageWorkEntity::getWorkStatus, OutageWorkEntity::getProcessInstanceId));

        if (outageWorkEntity.getWorkStatus() != null && outageWorkEntity.getWorkStatus() == 0) {
            flowableService.deleteProcessInstance(entity.getProcessInstanceId(), "取消工作");
        }


        return ApiResult.ok();
    }

    @Transactional
    public ApiResult<String> importFromExcel(MultipartFile file, Integer isTemp) {
        List<OutageWorkExcelVO> dataList;
        int errorCount = 0;
        try {
            dataList = SmartExcelUtil.readExcelWithHeaderValidation(file.getInputStream(), OutageWorkExcelVO.class);
            if (dataList.isEmpty()) {
                return ApiResult.okMsg("数据为空");
            }
            // 处理导入的数据，例如保存到数据库
            List<OutageWorkEntity> list = SmartBeanUtil.copyList(dataList, OutageWorkEntity.class);

            // 判断数据库中是否存在相同的数据
            List<String> entities = outageWorkDao.selectList(Wrappers.lambdaQuery(OutageWorkEntity.class)
                            .in(OutageWorkEntity::getInspectionContent, list.stream().map(OutageWorkEntity::getInspectionContent).toList())
                            .select(OutageWorkEntity::getInspectionContent))
                    .stream()
                    .map(OutageWorkEntity::getInspectionContent)
                    .toList();

            if (!entities.isEmpty()) {
                list.removeIf(entity -> entities.contains(entity.getInspectionContent()));
                errorCount = entities.size();
            }

            if (!list.isEmpty()) {
                Map<String, Integer> oldDataId = outageWorkDao.selectList(
                                Wrappers.lambdaQuery(OutageWorkEntity.class)
                                        .in(OutageWorkEntity::getInspectionContent, list.stream().map(OutageWorkEntity::getInspectionContent).toList())
                                        .select(OutageWorkEntity::getId, OutageWorkEntity::getInspectionContent)
                        )
                        .stream().collect(Collectors.toMap(OutageWorkEntity::getInspectionContent, OutageWorkEntity::getId));


                // 提前提取所有 inspectionContent 和对应的 substrings
                Map<String, List<String>> substringMap = list.stream()
                        .filter(entity -> StringUtils.isNotBlank(entity.getInspectionContent()))
                        .collect(Collectors.toMap(
                                OutageWorkEntity::getInspectionContent,
                                entity -> ReUtil.findAllGroup1("<(.*?)>", entity.getInspectionContent())
                        ));

                // 批量查询 whetherDebug
                List<Map<String, Object>> debugEntities = powerDebugService.listByContentAndTime(list);


                list.forEach(entity -> {
                    if (StringUtils.isNotBlank(entity.getInspectionContent())) {
                        List<String> substrings = substringMap.get(entity.getInspectionContent());
                        entity.setLineName(String.join("、", substrings));

                        Optional<Map<String, Object>> optional = debugEntities.stream().filter(it -> it.get("line_name").equals(entity.getInspectionContent()))
                                .filter(it -> {
                                    String string = entity.getStartDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                                    return it.get("start_time").equals(string);
                                })
                                .findFirst();

                        entity.setWhetherDebug(optional.isPresent() ? 1 : 0);
                    }
//                    setFlow(entity);
                    entity.setIsTemp(isTemp);
                    entity.setOldDataId(oldDataId.get(entity.getInspectionContent()));
                });


                SmartMybatisUtil.batchInsert(OutageWorkDao.class, list, 1000);

                String userId = LoginUtil.getLoginUser().getUserId().toString();
                for (OutageWorkEntity entity : list) {
                    startFlow(entity, userId);
                }
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            throw new BusinessException("数据格式存在问题，无法读取");
        }

        if (CollectionUtils.isEmpty(dataList)) {
            throw new BusinessException("数据为空");
        }

        return ApiResult.okMsg("成功导入" + dataList.size() + "条，" + (errorCount > 0 ? "失败" + errorCount + "条（检修内容有重复）" : ""));
    }

    /**
     * 批量删除
     *
     * @param idList
     * @return
     */
    @Transactional
    public ApiResult<String> batchDelete(List<Integer> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return ApiResult.ok();
        }
        List<OutageWorkEntity> entities = listByIds(idList);
        for (OutageWorkEntity entity : entities) {
            deleteFlow(entity);
        }
        outageWorkDao.deleteByIds(idList);
        return ApiResult.ok();
    }

    @Transactional
    public void deleteFlow(OutageWorkEntity entity) {
        if (entity.getProcessInstanceId() != null) {
            Object processes = flowableService.getVariable(entity.getProcessInstanceId(), "startedSubProcesses");

            if (processes == null) {
                processes = flowableService.getHistoricVariables(entity.getProcessInstanceId(), "startedSubProcesses");
                if (processes == null) {
                    return;
                }
            }
            @SuppressWarnings("unchecked")
            Map<String, String[]> processes1 = (Map<String, String[]>) processes;
            if (!processes1.isEmpty()) {
                flowableService.deleteProcess((processes1).keySet(), "deleted");
            }
            flowableService.deleteProcess(entity.getProcessInstanceId(), "deleted");
        }
    }

    /**
     * 单个删除
     */
    public ApiResult<String> delete(Integer id) {
        if (null == id) {
            return ApiResult.ok();
        }
        OutageWorkEntity entity = outageWorkDao.selectById(id);

        deleteFlow(entity);

        outageWorkDao.deleteById(id);
        return ApiResult.ok();
    }

    public OutageWorkVO getById(Integer id) {
        var form = new OutageWorkQueryForm();
        form.setPage(1L);
        form.setLimit(1L);
        form.setId(id);
        PageParam<OutageWorkEntity, OutageWorkQueryForm> page = new PageParam<>(form);
        List<OutageWorkVO> vos = outageWorkDao.queryPage(page, form);
        return vos.isEmpty() ? null : vos.get(0);
    }

    // 使用Stream API的简洁版本
    public static boolean hasCommonElementStream(List<Integer> list1, List<Integer> list2) {
        if (list1 == null || list2 == null || list1.isEmpty() || list2.isEmpty()) {
            return false;
        }
        Set<Integer> set = new HashSet<>(list2);
        return list1.stream().anyMatch(set::contains);
    }

    @Transactional
    public void underwayFlow(OutageWorkFlowVo vo, User user) {
        OutageWorkEntity outageWorkEntity = outageWorkDao.selectById(vo.getId());
        // 第一次走流程
        if (outageWorkEntity.getFlowId() == null) {
            setFlow(outageWorkEntity);
            vo.setStepNumber(outageWorkEntity.getStepNumber());
        }
        if (outageWorkEntity.getIsNewProject() != 1) {
            throw new BusinessException("未涉及新投异动");
        }
        if (outageWorkEntity.getStepNumber() != vo.getStepNumber()) {
            throw new BusinessException("流程已更新，请刷新");
        }
        if (outageWorkEntity.getStepNumber() < 1) {
            throw new BusinessException("当前流程已完成");
        }
        // 查询当前步骤
        Step step = stepService.getOne(Wrappers.lambdaQuery(Step.class).eq(Step::getFlowId, outageWorkEntity.getFlowId()).eq(Step::getStepNumber, vo.getStepNumber()));
        //  如果是第一步
        if (step.getStepNumber() == 1 && outageWorkEntity.getIsNewProject() == 1) {
            LocalDateTime dateTime = outageWorkEntity.getStartDate();
            LocalDate date = dateTime.toLocalDate();
            LocalDateTime dateOnly = date.atStartOfDay();
            // 定义格式（例如：yyyy-MM-dd HH:mm:ss）
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            String formattedDateTime = dateOnly.format(formatter);

            DistributionDeviceInfoEntity distributionDeviceInfoEntity = distributionDeviceInfoDao.selectOne(Wrappers.lambdaQuery(DistributionDeviceInfoEntity.class).eq(DistributionDeviceInfoEntity::getPlanWorkTime, formattedDateTime).eq(DistributionDeviceInfoEntity::getLineName, outageWorkEntity.getLineName()));
            if (distributionDeviceInfoEntity != null) {
                distributionDeviceInfoEntity.setOutageId(outageWorkEntity.getId());
                distributionDeviceInfoDao.updateById(distributionDeviceInfoEntity);
            } else {
                throw new BusinessException("资料未收集");
            }
        }

        // 验证角色是否有权限走该流程
        String strIds = step.getStrIds();
        if (StringUtils.isNotBlank(strIds)) {
            String[] split = strIds.split(",");
            List<Integer> strIdList = Arrays.stream(split).map(Integer::parseInt).collect(Collectors.toList());
            List<Integer> roleIds = user.getRoles().stream().map(Role::getRoleId).collect(Collectors.toList());
            if (!hasCommonElementStream(strIdList, roleIds)) {
                throw new BusinessException("没有当前流程权限");
            }
        }
        // 存入停电步骤表
        StepTable stepTable = new StepTable(outageWorkEntity, step, user.getUserId());
        stepTableService.save(stepTable);
        // 存入字段
        if (StringUtils.isNotBlank(step.getFieldIds())) {
            List<OutageWorkFieldVo> fieldList = vo.getFieldList();
            for (OutageWorkFieldVo outageWorkFieldVo : fieldList) {
                Field field = fieldService.getById(outageWorkFieldVo.getFieldId());
                StepTableField stepTableField = new StepTableField(stepTable.getId(), field, outageWorkFieldVo.getFieldValue());
                stepTableFieldService.save(stepTableField);
            }
        }
        Step nextStep = stepService.getOne(Wrappers.lambdaQuery(Step.class).eq(Step::getFlowId, outageWorkEntity.getFlowId()).eq(Step::getStepNumber, vo.getStepNumber() + 1));
        if (nextStep == null) {
            outageWorkEntity.setStepNumber(-1);
        } else if (nextStep.getStepNumber() == 5) {
            DistributionDeviceInfoEntity distributionDeviceInfoEntity = distributionDeviceInfoDao.selectOne(Wrappers.lambdaQuery(DistributionDeviceInfoEntity.class).eq(DistributionDeviceInfoEntity::getOutageId, outageWorkEntity.getId()));
           /* if (distributionDeviceInfoEntity != null && distributionDeviceInfoEntity.getWhetherDebug() != 1) {
                outageWorkEntity.setStepNumber(6);
            } else {
                outageWorkEntity.setStepNumber(5);
            }*/

        } else {
            outageWorkEntity.setStepNumber(vo.getStepNumber() + 1);
        }
        outageWorkDao.updateById(outageWorkEntity);

    }

    public List<StepTableVo> getFlowList(Integer id) {
        OutageWorkEntity outageWorkEntity = outageWorkDao.selectById(id);
        Integer whetherDebug = null;
        DistributionDeviceInfoEntity distributionDeviceInfoEntity = distributionDeviceInfoDao.selectOne(Wrappers.lambdaQuery(DistributionDeviceInfoEntity.class).eq(DistributionDeviceInfoEntity::getOutageId, outageWorkEntity.getId()));
        /*if (distributionDeviceInfoEntity != null && distributionDeviceInfoEntity.getWhetherDebug() != 1) {
            whetherDebug = 5;
        }*/

        List<StepTableVo> flowList = stepTableService.getFlowList(id, whetherDebug);
        flowList.forEach(item -> {
            if (item.getStepTableId() != null) {
                item.setFieldlist(stepTableFieldService.getFlowFieldList(item.getStepTableId()));
            }
        });
        return flowList;
    }

    public Map<String, Object> getFieldList(Integer id) {
        OutageWorkEntity outageWorkEntity = outageWorkDao.selectById(id);
        if (outageWorkEntity.getFlowId() == null) {
            setFlow(outageWorkEntity);
        }
        Map<String, Object> result = new HashMap<>();
        Step step = stepService.getOne(Wrappers.lambdaQuery(Step.class).eq(Step::getFlowId, outageWorkEntity.getFlowId()).eq(Step::getStepNumber, outageWorkEntity.getStepNumber()));
        result.put("stepTitle", step.getStepTitle());
        result.put("fieldList", new ArrayList<>());
        if (StringUtils.isNotBlank(step.getFieldIds())) {
            List<String> collect = Arrays.stream(step.getFieldIds().split(",")).collect(Collectors.toList());
            List<Field> fields = fieldService.listByIds(collect);
            result.put("fieldList", fields);
        }
        return result;
    }

    public void cancel(OutageWorkCanelVo vo) {
        OutageWorkEntity outageWorkEntity = new OutageWorkEntity();
        outageWorkEntity.setId(vo.getId());
        outageWorkEntity.setCancelStatus(vo.getCancelStatus());
        outageWorkDao.updateById(outageWorkEntity);

    }

    public OutageWorkVO cancelInfo(OutageWorkCancelInfoVO outageWorkCancelInfoVO) {
        return outageWorkDao.cancelInfo(outageWorkCancelInfoVO);
    }

    public DistributionDeviceInfoUpdateVO getDisttributionDevice(Integer id) {
        DistributionDeviceInfoUpdateVO vo = new DistributionDeviceInfoUpdateVO();
//        OutageWorkVO byId = getById(id);
        var form = new DistributionDeviceInfoQueryForm();
        form.setPage(1L);
        form.setLimit(1L);
        form.setOutageId(id);
//        form.setLineName(byId.getLineName());
        PageParam<DistributionDeviceInfoEntity, DistributionDeviceInfoQueryForm> page = new PageParam<>(form);
        distributionDeviceInfoDao.queryPage(page, form).stream().findFirst().ifPresent(vo::setLeft);
//        powerDebugManager.getOneOpt(Wrappers.lambdaQuery(PowerDebugEntity.class).eq(PowerDebugEntity::getDdiId, id).last("limit 1")).ifPresent(vo::setRight);
        return vo;

    }

    public long getOverdueWork() {
        return flowableService.getOverdueWork("图模归档");
    }

    public List<OutageWorkEntity> listRel(Collection<String> ids) {
        return baseMapper.listRel(ids);
    }
}
