package com.eleadmin.powerdebug.controller;

import com.eleadmin.common.core.annotation.Operation;
import com.eleadmin.common.core.annotation.Tag;
import com.eleadmin.common.core.utils.SmartExcelUtil;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.BaseController;
import com.eleadmin.common.core.web.PageResult;
import com.eleadmin.powerdebug.domain.entity.PowerDebugEntity;
import com.eleadmin.powerdebug.domain.form.PowerDebugAddForm;
import com.eleadmin.powerdebug.domain.form.PowerDebugQueryForm;
import com.eleadmin.powerdebug.domain.form.PowerDebugUpdateForm;
import com.eleadmin.powerdebug.domain.vo.PowerDebugExcelVO;
import com.eleadmin.powerdebug.service.PowerDebugService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * 电力设备调试工作记录表 Controller
 *
 * @Author 王益亭
 * @Date 2025-03-03 10:05:18
 * @Copyright gxrd
 */

@RestController
@Tag(name = "电力设备调试工作记录表")
public class PowerDebugController extends BaseController {

    @Resource
    private PowerDebugService powerDebugService;

    @Operation(summary = "分页查询 @author 王益亭")
    @PostMapping("/api/powerDebug/queryPage")
    public ApiResult<PageResult> queryPage(@RequestBody @Valid PowerDebugQueryForm queryForm) {
        return success(powerDebugService.querySimplePage(queryForm));
    }

    @Operation(summary = "分页查询 @author 王益亭")
    @PostMapping("/api/powerDebug/list")
    public ApiResult<List<PowerDebugEntity>> list(@RequestBody PowerDebugQueryForm queryForm) {
        return success(powerDebugService.lambdaQuery().eq(PowerDebugEntity::getOutageId, queryForm.getOutageId()).list());
    }

    @Operation(summary = "根据ID查询 @author 王益亭")
    @PostMapping("/api/powerDebug/{id}")
//    // @SaCheckPermission("distributionDeviceInfo:query")
    @PreAuthorize("hasAuthority('powerDebug:query')")
    public ApiResult getById(@PathVariable Integer id) {
        return success(powerDebugService.getById(id));
    }

    @Operation(summary = "导入Excel @author 王益亭")
    @PostMapping("/api/powerDebug/importExcel")
//    // @SaCheckPermission("powerDebug:import")
    @PreAuthorize("hasAuthority('powerDebug:import')")
    public ApiResult<String> importExcel(@RequestParam("file") MultipartFile file) {
        return powerDebugService.importFromExcel(file);
    }

    @Operation(summary = "电力设备调试工作记录模板 @author 王益亭")
    @GetMapping("/api/powerDebug/model")
    // @SaCheckPermission("powerDebug:import")
    @PreAuthorize("hasAuthority('powerDebug:import')")
    public void model(HttpServletResponse response) throws IOException {
        SmartExcelUtil.exportExcel(response, "电力设备调试工作记录表","sheet", PowerDebugExcelVO.class, List.of(new PowerDebugExcelVO()));
    }

    @Operation(summary = "添加 @author 王益亭")
    @PostMapping("/api/powerDebug/add")
    // @SaCheckPermission("powerDebug:add")
    @PreAuthorize("hasAuthority('powerDebug:add')")
    public ApiResult<String> add(@RequestBody @Valid PowerDebugAddForm addForm) {
        return powerDebugService.add(addForm);
    }

    @Operation(summary = "更新 @author 王益亭")
    @PostMapping("/api/powerDebug/update")
    // @SaCheckPermission("powerDebug:update")
    @PreAuthorize("hasAuthority('powerDebug:update')")
    public ApiResult<String> update(@RequestBody @Valid PowerDebugUpdateForm updateForm) {
        return powerDebugService.update(updateForm);
    }

    @Operation(summary = "批量删除 @author 王益亭")
    @PostMapping("/api/powerDebug/batchDelete")
    // @SaCheckPermission("powerDebug:delete")
    @PreAuthorize("hasAuthority('powerDebug:remove')")
    public ApiResult<String> batchDelete(@RequestBody List<Integer> idList) {
        return powerDebugService.batchDelete(idList);
    }

    @Operation(summary = "单个删除 @author 王益亭")
    @GetMapping("/api/powerDebug/delete/{id}")
    @PreAuthorize("hasAuthority('powerDebug:remove')")
    // @SaCheckPermission("powerDebug:delete")
    public ApiResult<String> batchDelete(@PathVariable Integer id) {
        return powerDebugService.delete(id);
    }

}
